@@ -38,8 +38,8 @@
 #include <cstdlib>
 #include <cassert>
 
-#include "PeerMessageUtil.h"
-#include "Util.h"
+#include "bittorrent_helper.h"
+#include "util.h"
 #include "message.h"
 #include "DlAbortEx.h"
 #include "MessageDigestHelper.h"
@@ -47,13 +47,13 @@
 #include "Logger.h"
 #include "Peer.h"
 #include "Piece.h"
-#include "BtContext.h"
 #include "PieceStorage.h"
 #include "BtMessageDispatcher.h"
 #include "BtMessageFactory.h"
 #include "BtRequestFactory.h"
 #include "PeerConnection.h"
 #include "StringFormat.h"
+#include "DownloadContext.h"
 
 namespace aria2 {
 
@@ -67,16 +67,19 @@ void BtPieceMessage::setBlock(const unsigned char* block, size_t blockLength) {
 }
 
 BtPieceMessageHandle BtPieceMessage::create(const unsigned char* data, size_t dataLength) {
-  PeerMessageUtil::assertPayloadLengthGreater(9, dataLength, NAME);
-  PeerMessageUtil::assertID(ID, data, NAME);
+  bittorrent::assertPayloadLengthGreater(9, dataLength, NAME);
+  bittorrent::assertID(ID, data, NAME);
   BtPieceMessageHandle message(new BtPieceMessage());
-  message->setIndex(PeerMessageUtil::getIntParam(data, 1));
-  message->setBegin(PeerMessageUtil::getIntParam(data, 5));
+  message->setIndex(bittorrent::getIntParam(data, 1));
+  message->setBegin(bittorrent::getIntParam(data, 5));
   message->setBlock(data+9, dataLength-9);
   return message;
 }
 
 void BtPieceMessage::doReceivedAction() {
+  if(_metadataGetMode) {
+    return;
+  }
   RequestSlot slot = dispatcher->getOutstandingRequest(index,
 						       begin,
 						       blockLength);
@@ -85,13 +88,13 @@ void BtPieceMessage::doReceivedAction() {
     peer->snubbing(false);
     peer->updateLatency(slot.getLatencyInMillis());
     PieceHandle piece = pieceStorage->getPiece(index);
-    off_t offset = (off_t)index*btContext->getPieceLength()+begin;
+    off_t offset = (off_t)index*_downloadContext->getPieceLength()+begin;
     logger->debug(MSG_PIECE_RECEIVED,
 		  cuid, index, begin, blockLength, offset, slot.getBlockIndex());
     pieceStorage->getDiskAdaptor()->writeData(block, blockLength, offset);
     piece->completeBlock(slot.getBlockIndex());
     logger->debug(MSG_PIECE_BITFIELD, cuid,
-		  Util::toHex(piece->getBitfield(),
+		  util::toHex(piece->getBitfield(),
 			      piece->getBitfieldLength()).c_str());
     piece->updateHash(begin, block, blockLength);
     dispatcher->removeOutstandingRequest(slot);
@@ -120,10 +123,10 @@ const unsigned char* BtPieceMessage::getMessageHeader() {
      * total: 13bytes
      */
     msgHeader = new unsigned char[MESSAGE_HEADER_LENGTH];
-    PeerMessageUtil::createPeerMessageString(msgHeader, MESSAGE_HEADER_LENGTH,
-					     9+blockLength, ID);
-    PeerMessageUtil::setIntParam(&msgHeader[5], index);
-    PeerMessageUtil::setIntParam(&msgHeader[9], begin);
+    bittorrent::createPeerMessageString(msgHeader, MESSAGE_HEADER_LENGTH,
+					9+blockLength, ID);
+    bittorrent::setIntParam(&msgHeader[5], index);
+    bittorrent::setIntParam(&msgHeader[9], begin);
   }
   return msgHeader;
 }
@@ -143,7 +146,7 @@ void BtPieceMessage::send() {
     getMessageHeader();
     peerConnection->sendMessage(msgHeader, getMessageHeaderLength());
     off_t pieceDataOffset =
-      (off_t)index*btContext->getPieceLength()+begin;
+      (off_t)index*_downloadContext->getPieceLength()+begin;
     size_t writtenLength = sendPieceData(pieceDataOffset, blockLength);
     logger->debug("msglength = %lu bytes",
 		  static_cast<unsigned long>(getMessageHeaderLength()+
@@ -168,20 +171,21 @@ size_t BtPieceMessage::sendPieceData(off_t offset, size_t length) const {
 }
 
 std::string BtPieceMessage::toString() const {
-  return strconcat(NAME, " index=", Util::itos(index), ", begin=",
-		   Util::itos(begin), ", length=", Util::itos(blockLength));
+  return strconcat(NAME, " index=", util::itos(index), ", begin=",
+		   util::itos(begin), ", length=", util::itos(blockLength));
 }
 
 bool BtPieceMessage::checkPieceHash(const PieceHandle& piece) {
   if(piece->isHashCalculated()) {
     logger->debug("Hash is available!! index=%lu",
 		  static_cast<unsigned long>(piece->getIndex()));
-    return piece->getHashString() == btContext->getPieceHash(piece->getIndex());
+    return
+      piece->getHashString()==_downloadContext->getPieceHash(piece->getIndex());
   } else {
-    off_t offset = (off_t)piece->getIndex()*btContext->getPieceLength();
+    off_t offset = (off_t)piece->getIndex()*_downloadContext->getPieceLength();
     
     return MessageDigestHelper::staticSHA1Digest(pieceStorage->getDiskAdaptor(), offset, piece->getLength())
-      == btContext->getPieceHash(piece->getIndex());
+      == _downloadContext->getPieceHash(piece->getIndex());
   }
 }
 
@@ -203,7 +207,7 @@ void BtPieceMessage::erasePieceOnDisk(const PieceHandle& piece) {
   size_t BUFSIZE = 4096;
   unsigned char buf[BUFSIZE];
   memset(buf, 0, BUFSIZE);
-  off_t offset = (off_t)piece->getIndex()*btContext->getPieceLength();
+  off_t offset = (off_t)piece->getIndex()*_downloadContext->getPieceLength();
   div_t res = div(piece->getLength(), BUFSIZE);
   for(int i = 0; i < res.quot; ++i) {
     pieceStorage->getDiskAdaptor()->writeData(buf, BUFSIZE, offset);
@@ -255,4 +259,10 @@ void BtPieceMessage::onCancelSendingPieceEvent
   } 
 }
 
+void BtPieceMessage::setDownloadContext
+(const SharedHandle<DownloadContext>& downloadContext)
+{
+  _downloadContext = downloadContext;
+}
+
 } // namespace aria2
