@@ -33,7 +33,6 @@
  */
 /* copyright --> */
 #include "BtPostDownloadHandler.h"
-#include "DefaultBtContext.h"
 #include "prefs.h"
 #include "RequestGroup.h"
 #include "Option.h"
@@ -42,17 +41,22 @@
 #include "File.h"
 #include "PieceStorage.h"
 #include "DiskAdaptor.h"
-#include "Util.h"
+#include "util.h"
 #include "ContentTypeRequestGroupCriteria.h"
 #include "Exception.h"
+#include "DownloadContext.h"
+#include "download_helper.h"
 
 namespace aria2 {
 
 BtPostDownloadHandler::BtPostDownloadHandler()
 {
   SharedHandle<RequestGroupCriteria> cri
-    (new ContentTypeRequestGroupCriteria(DownloadHandlerConstants::getBtContentTypes(),
-					 DownloadHandlerConstants::getBtExtensions()));
+    (new ContentTypeRequestGroupCriteria
+     (DownloadHandlerConstants::getBtContentTypes().begin(),
+      DownloadHandlerConstants::getBtContentTypes().end(),
+      DownloadHandlerConstants::getBtExtensions().begin(),
+      DownloadHandlerConstants::getBtExtensions().end()));
   setCriteria(cri);
 }
 
@@ -62,30 +66,20 @@ void BtPostDownloadHandler::getNextRequestGroups
 (std::deque<SharedHandle<RequestGroup> >& groups,
  RequestGroup* requestGroup)
 {
-  const Option* op = requestGroup->getOption();
   _logger->debug("Generating RequestGroups for Torrent file %s",
-		 requestGroup->getFilePath().c_str());
-  RequestGroupHandle rg(new RequestGroup(op, std::deque<std::string>()));
-
+		 requestGroup->getFirstFilePath().c_str());
   std::string content;
   try {
     requestGroup->getPieceStorage()->getDiskAdaptor()->openExistingFile();
-    content = Util::toString(requestGroup->getPieceStorage()->getDiskAdaptor());
+    content = util::toString(requestGroup->getPieceStorage()->getDiskAdaptor());
     requestGroup->getPieceStorage()->getDiskAdaptor()->closeFile();    
   } catch(Exception& e) {
     requestGroup->getPieceStorage()->getDiskAdaptor()->closeFile();
     throw;
   }
-  DefaultBtContextHandle btContext(new DefaultBtContext());
-  btContext->loadFromMemory(content, File(requestGroup->getFilePath()).getBasename());
-  if(op->defined(PREF_PEER_ID_PREFIX)) {
-    btContext->setPeerIdPrefix(op->get(PREF_PEER_ID_PREFIX));
-  }
-  btContext->setDir(requestGroup->getDownloadContext()->getDir());
-  rg->setDownloadContext(btContext);
-  btContext->setOwnerRequestGroup(rg.get());
-  
-  groups.push_back(rg);
+  createRequestGroupForBitTorrent(groups, requestGroup->getOption(),
+				  std::deque<std::string>(),
+				  content);
 }
 
 } // namespace aria2
