@@ -37,11 +37,9 @@
 
 #include "common.h"
 
-#include <string>
 #include <map>
 
 #include "SharedHandle.h"
-#include "HandleRegistry.h"
 
 namespace aria2 {
 
@@ -50,56 +48,69 @@ class PieceStorage;
 class BtAnnounce;
 class BtRuntime;
 class BtProgressInfoFile;
-class BtContext;
+class DownloadContext;
 
-typedef HandleRegistry<std::string, PeerStorage> PeerStorageMap;
-typedef HandleRegistry<std::string, PieceStorage> PieceStorageMap;
-typedef HandleRegistry<std::string, BtAnnounce> BtAnnounceMap;
-typedef HandleRegistry<std::string, BtRuntime> BtRuntimeMap;
-typedef HandleRegistry<std::string, BtProgressInfoFile> BtProgressInfoFileMap;
-typedef HandleRegistry<std::string, BtContext>  BtContextMap;
+struct BtObject {
+  SharedHandle<DownloadContext> _downloadContext;
+  SharedHandle<PieceStorage> _pieceStorage;
+  SharedHandle<PeerStorage> _peerStorage;
+  SharedHandle<BtAnnounce> _btAnnounce;
+  SharedHandle<BtRuntime> _btRuntime;
+  SharedHandle<BtProgressInfoFile> _btProgressInfoFile;
+
+  BtObject(const SharedHandle<DownloadContext>& downloadContext,
+	   const SharedHandle<PieceStorage>& pieceStorage,
+	   const SharedHandle<PeerStorage>& peerStorage,
+	   const SharedHandle<BtAnnounce>& btAnnounce,
+	   const SharedHandle<BtRuntime>& btRuntime,
+	   const SharedHandle<BtProgressInfoFile>& btProgressInfoFile):
+    _downloadContext(downloadContext),
+    _pieceStorage(pieceStorage),
+    _peerStorage(peerStorage),
+    _btAnnounce(btAnnounce),
+    _btRuntime(btRuntime),
+    _btProgressInfoFile(btProgressInfoFile) {}
+
+  BtObject() {}
+
+  bool isNull() const
+  {
+    return _downloadContext.isNull() &&
+      _pieceStorage.isNull() &&
+      _peerStorage.isNull() &&
+      _btAnnounce.isNull() &&
+      _btRuntime.isNull() &&
+      _btProgressInfoFile.isNull();
+  }
+};
 
 class BtRegistry {
 private:
-  BtContextMap btContextMap;
-  PeerStorageMap peerStorageMap;
-  PieceStorageMap pieceStorageMap;
-  BtAnnounceMap btAnnounceMap;
-  BtRuntimeMap btRuntimeMap;
-  BtProgressInfoFileMap btProgressInfoFileMap;
+  std::map<int32_t, BtObject> _pool;
 public:
-  BtRegistry();
-  
-  SharedHandle<BtContext> getBtContext(const std::string& key);
-  void registerBtContext(const std::string& key,
-			 const SharedHandle<BtContext>& btContext);
+  SharedHandle<DownloadContext>
+  getDownloadContext(int32_t gid) const;
 
-  SharedHandle<PeerStorage> getPeerStorage(const std::string& key);
-  void registerPeerStorage(const std::string& key,
-			   const SharedHandle<PeerStorage>& peer);
-				  
-  SharedHandle<PieceStorage> getPieceStorage(const std::string& key);
-  void registerPieceStorage(const std::string& key,
-			    const SharedHandle<PieceStorage>& pieceStorage);
+  SharedHandle<DownloadContext>
+  getDownloadContext(const std::string& infoHash) const;
 
-  SharedHandle<BtRuntime> getBtRuntime(const std::string& key);
-  void registerBtRuntime(const std::string& key,
-			 const SharedHandle<BtRuntime>& btRuntime);
+  void put(int32_t gid, const BtObject& obj);
 
-  SharedHandle<BtAnnounce> getBtAnnounce(const std::string& key);
-  void registerBtAnnounce(const std::string& key,
-				 const SharedHandle<BtAnnounce>& btAnnounce);
+  BtObject get(int32_t gid) const;
 
-  SharedHandle<BtProgressInfoFile>
-  getBtProgressInfoFile(const std::string& key);
-  void registerBtProgressInfoFile(const std::string& key,
-				  const SharedHandle<BtProgressInfoFile>& file);
+  template<typename OutputIterator>
+  OutputIterator getAllDownloadContext(OutputIterator dest)
+  {
+    for(std::map<int32_t, BtObject>::const_iterator i = _pool.begin();
+	i != _pool.end(); ++i) {
+      *dest++ = (*i).second._downloadContext;
+    }
+    return dest;
+  }
 
-  std::deque<SharedHandle<BtContext> > getAllBtContext();
+  void removeAll();
 
-  void unregisterAll();
-
-  void unregister(const std::string& key);
+  bool remove(int32_t gid);
 };
 
 } // namespace aria2
