@@ -33,14 +33,21 @@
  */
 /* copyright --> */
 #include "CheckIntegrityEntry.h"
-#include "Command.h"
-#include "RequestGroup.h"
 #include "IteratableValidator.h"
+#include "RequestGroup.h"
+#include "PieceStorage.h"
+#include "DiskAdaptor.h"
+#include "prefs.h"
+#include "FileAllocationEntry.h"
+#include "DownloadEngine.h"
+#include "Option.h"
+#include "DownloadContext.h"
+
+namespace aria2 {
 
 CheckIntegrityEntry::CheckIntegrityEntry(RequestGroup* requestGroup,
 					 Command* nextCommand):
-  RequestGroupEntry(requestGroup, nextCommand),
-  _validator(0)
+  RequestGroupEntry(requestGroup, nextCommand)
 {}
 
 CheckIntegrityEntry::~CheckIntegrityEntry() {}
@@ -50,7 +57,7 @@ void CheckIntegrityEntry::validateChunk()
   _validator->validateChunk();
 }
 
-int64_t CheckIntegrityEntry::getTotalLength()
+uint64_t CheckIntegrityEntry::getTotalLength()
 {
   if(_validator.isNull()) {
     return 0;
@@ -59,7 +66,7 @@ int64_t CheckIntegrityEntry::getTotalLength()
   }
 }
 
-int64_t CheckIntegrityEntry::getCurrentLength()
+off_t CheckIntegrityEntry::getCurrentLength()
 {
   if(_validator.isNull()) {
     return 0;
@@ -72,3 +79,27 @@ bool CheckIntegrityEntry::finished()
 {
   return _validator->finished();
 }
+
+void CheckIntegrityEntry::cutTrailingGarbage()
+{
+  _requestGroup->getPieceStorage()->getDiskAdaptor()->cutTrailingGarbage();
+}
+
+void CheckIntegrityEntry::proceedFileAllocation
+(std::deque<Command*>& commands,
+ const SharedHandle<FileAllocationEntry>& entry,
+ DownloadEngine* e)
+{
+  if(_requestGroup->needsFileAllocation()) {
+    e->_fileAllocationMan->pushEntry(entry);
+  } else {
+    entry->prepareForNextAction(commands, e);
+  }
+  // Disable directIO when fallocation() is going to be used.
+  if(_requestGroup->getOption()->get(PREF_FILE_ALLOCATION) == V_FALLOC) {
+    entry->disableDirectIO();
+  }
+}
+
+
+} // namespace aria2
