@@ -34,12 +34,14 @@
 /* copyright --> */
 #include "ChecksumCheckIntegrityEntry.h"
 #include "RequestGroup.h"
-#include "Command.h"
-#include "SingleFileDownloadContext.h"
+#include "DownloadContext.h"
+#include "FileEntry.h"
 #include "IteratableChecksumValidator.h"
 #include "DownloadEngine.h"
 #include "PieceStorage.h"
 
+namespace aria2 {
+
 ChecksumCheckIntegrityEntry::ChecksumCheckIntegrityEntry(RequestGroup* requestGroup, Command* nextCommand):
   CheckIntegrityEntry(requestGroup, nextCommand) {}
 
@@ -47,25 +49,26 @@ ChecksumCheckIntegrityEntry::~ChecksumCheckIntegrityEntry() {}
 
 bool ChecksumCheckIntegrityEntry::isValidationReady()
 {
-  SingleFileDownloadContextHandle dctx = _requestGroup->getDownloadContext();
-  return !dctx.isNull() && dctx->getChecksum().size() > 0 &&
-    dctx->getChecksumHashAlgo().size() > 0;
+  return !_requestGroup->getDownloadContext()->getChecksum().empty() &&
+    !_requestGroup->getDownloadContext()->getChecksumHashAlgo().empty();
 }
 
 void ChecksumCheckIntegrityEntry::initValidator()
 {
-  _validator = new IteratableChecksumValidator(_requestGroup->getDownloadContext(),
-					       _requestGroup->getPieceStorage());
+  _validator.reset(new IteratableChecksumValidator
+		   (_requestGroup->getDownloadContext(),
+		    _requestGroup->getPieceStorage()));
   _validator->init();
 }
 
-Commands ChecksumCheckIntegrityEntry::onDownloadFinished(DownloadEngine* e)
-{
-  return Commands();
-}
+void
+ChecksumCheckIntegrityEntry::onDownloadFinished(std::deque<Command*>& commands,
+						DownloadEngine* e)
+{}
 
-Commands ChecksumCheckIntegrityEntry::onDownloadIncomplete(DownloadEngine* e)
-{
-  return Commands();
-}
+void
+ChecksumCheckIntegrityEntry::onDownloadIncomplete(std::deque<Command*>& commands,
+						  DownloadEngine* e)
+{}
 
+} // namespace aria2
