@@ -33,17 +33,20 @@
  */
 /* copyright --> */
 #include "DHTBucket.h"
-#include "DHTUtil.h"
-#include "DHTNode.h"
-#include "LogFactory.h"
-#include "Logger.h"
-#include "Util.h"
-#include "DHTConstants.h"
-#include "a2functional.h"
+
 #include <cstring>
 #include <cassert>
 #include <algorithm>
 
+#include "DHTNode.h"
+#include "LogFactory.h"
+#include "Logger.h"
+#include "util.h"
+#include "DHTConstants.h"
+#include "a2functional.h"
+#include "bittorrent_helper.h"
+#include "bitfield.h"
+
 namespace aria2 {
 
 DHTBucket::DHTBucket(size_t prefixLength,
@@ -71,10 +74,10 @@ DHTBucket::~DHTBucket() {}
 void DHTBucket::getRandomNodeID(unsigned char* nodeID) const
 {
   if(_prefixLength == 0) {
-    DHTUtil::generateRandomKey(nodeID);
+    bittorrent::generateRandomKey(nodeID);
   } else {
     size_t lastByteIndex = (_prefixLength-1)/8;
-    DHTUtil::generateRandomKey(nodeID);
+    bittorrent::generateRandomKey(nodeID);
     memcpy(nodeID, _min, lastByteIndex+1);
   }
 }
@@ -89,25 +92,16 @@ bool DHTBucket::isInRange(const unsigned char* nodeID) const
   return isInRange(nodeID, _max, _min);
 }
 
+// Returns true if nodeID is in [min, max] (inclusive).
 bool DHTBucket::isInRange(const unsigned char* nodeID,
 			  const unsigned char* max,
 			  const unsigned char* min) const
 {
-  for(size_t i = 0; i < DHT_ID_LENGTH; ++i) {
-    if(nodeID[i] < min[i]) {
-      return false;
-    } else if(min[i] < nodeID[i]) {
-      break;
-    }
-  }
-  for(size_t i = 0; i < DHT_ID_LENGTH; ++i) {
-    if(max[i] < nodeID[i]) {
-      return false;
-    } else if(nodeID[i] < max[i]) {
-      break;
-    }
-  }
-  return true;
+  return
+    !std::lexicographical_compare(&nodeID[0], &nodeID[DHT_ID_LENGTH],
+				  &min[0], &min[DHT_ID_LENGTH]) &&
+    !std::lexicographical_compare(&max[0], &max[DHT_ID_LENGTH],
+				  &nodeID[0], &nodeID[DHT_ID_LENGTH]);
 }
 
 bool DHTBucket::addNode(const SharedHandle<DHTNode>& node)
@@ -186,60 +180,39 @@ SharedHandle<DHTBucket> DHTBucket::split()
 
   unsigned char rMax[DHT_ID_LENGTH];
   memcpy(rMax, _max, DHT_ID_LENGTH);
-  DHTUtil::flipBit(rMax, DHT_ID_LENGTH, _prefixLength);
+  bitfield::flipBit(rMax, DHT_ID_LENGTH, _prefixLength);
   unsigned char rMin[DHT_ID_LENGTH];
   memcpy(rMin, _min, DHT_ID_LENGTH);
 
-  DHTUtil::flipBit(_min, DHT_ID_LENGTH, _prefixLength);
+  bitfield::flipBit(_min, DHT_ID_LENGTH, _prefixLength);
 
-  size_t rPrefixLength;
-  size_t lPrefixLength;
-  if(isInRange(_localNode->getID(), rMax, rMin)) {
-    rPrefixLength = _prefixLength+1;
-    lPrefixLength = _prefixLength;
-  } else if(isInRange(_localNode->getID(), _max, _min)) {
-    rPrefixLength = _prefixLength;
-    lPrefixLength = _prefixLength+1;
-  } else {
-    rPrefixLength = _prefixLength;
-    lPrefixLength = _prefixLength;
-  }
-
-  SharedHandle<DHTBucket> rBucket(new DHTBucket(rPrefixLength,
+  ++_prefixLength;
+  SharedHandle<DHTBucket> rBucket(new DHTBucket(_prefixLength,
 						rMax, rMin, _localNode));
-  std::deque<SharedHandle<DHTNode> > tempNodes = _nodes;
-  for(std::deque<SharedHandle<DHTNode> >::iterator i = tempNodes.begin();
-      i != tempNodes.end();) {
+
+  std::deque<SharedHandle<DHTNode> > lNodes;
+  for(std::deque<SharedHandle<DHTNode> >::iterator i = _nodes.begin();
+      i != _nodes.end(); ++i) {
     if(rBucket->isInRange(*i)) {
       assert(rBucket->addNode(*i));
-      i = tempNodes.erase(i);
     } else {
-      ++i;
-    }
+      lNodes.push_back(*i);
+    }      
   }
-  _prefixLength = lPrefixLength;
-  _nodes = tempNodes;
+  _nodes = lNodes;
   // TODO create toString() and use it.
-  _logger->debug("New bucket. Range:%s-%s",
- 		 Util::toHex(rBucket->getMinID(), DHT_ID_LENGTH).c_str(),
- 		 Util::toHex(rBucket->getMaxID(), DHT_ID_LENGTH).c_str());
-  _logger->debug("Existing bucket. Range:%s-%s",
-		 Util::toHex(getMinID(), DHT_ID_LENGTH).c_str(),
-		 Util::toHex(getMaxID(), DHT_ID_LENGTH).c_str());
+  _logger->debug("New bucket. prefixLength=%u, Range:%s-%s",
+		 static_cast<unsigned int>(rBucket->getPrefixLength()),
+ 		 util::toHex(rBucket->getMinID(), DHT_ID_LENGTH).c_str(),
+ 		 util::toHex(rBucket->getMaxID(), DHT_ID_LENGTH).c_str());
+  _logger->debug("Existing bucket. prefixLength=%u, Range:%s-%s",
+		 static_cast<unsigned int>(_prefixLength),
+		 util::toHex(getMinID(), DHT_ID_LENGTH).c_str(),
+		 util::toHex(getMaxID(), DHT_ID_LENGTH).c_str());
 
   return rBucket;
 }
 
-size_t DHTBucket::countNode() const
-{
-  return _nodes.size();
-}
-
-const std::deque<SharedHandle<DHTNode> >& DHTBucket::getNodes() const
-{
-  return _nodes;
-}
-
 void DHTBucket::getGoodNodes(std::deque<SharedHandle<DHTNode> >& goodNodes) const
 {
   goodNodes = _nodes;
@@ -301,9 +274,4 @@ SharedHandle<DHTNode> DHTBucket::getLRUQuestionableNode() const
   }
 }
 
-const std::deque<SharedHandle<DHTNode> >& DHTBucket::getCachedNodes() const
-{
-  return _cachedNodes;
-}
-
 } // namespace aria2
