@@ -39,7 +39,7 @@
 #include "prefs.h"
 #include "message.h"
 #include "BtHandshakeMessage.h"
-#include "Util.h"
+#include "util.h"
 #include "BtKeepAliveMessage.h"
 #include "BtChokeMessage.h"
 #include "BtUnchokeMessage.h"
@@ -54,7 +54,7 @@
 #include "DHTNode.h"
 #include "Peer.h"
 #include "Piece.h"
-#include "BtContext.h"
+#include "DownloadContext.h"
 #include "PieceStorage.h"
 #include "PeerStorage.h"
 #include "BtRuntime.h"
@@ -68,28 +68,34 @@
 #include "StringFormat.h"
 #include "RequestGroup.h"
 #include "RequestGroupMan.h"
+#include "bittorrent_helper.h"
+#include "UTMetadataRequestFactory.h"
+#include "UTMetadataRequestTracker.h"
 
 namespace aria2 {
 
 DefaultBtInteractive::DefaultBtInteractive
-(const SharedHandle<BtContext>& btContext, const SharedHandle<Peer>& peer)
+(const SharedHandle<DownloadContext>& downloadContext,
+ const SharedHandle<Peer>& peer)
   :
-  _btContext(btContext),
+  _downloadContext(downloadContext),
   peer(peer),
+  _metadataGetMode(false),
   logger(LogFactory::getInstance()),
   allowedFastSetSize(10),
   keepAliveInterval(120),
   _utPexEnabled(false),
   _dhtEnabled(false),
-  _numReceivedMessage(0)
+  _numReceivedMessage(0),
+  _maxOutstandingRequest(DEFAULT_MAX_OUTSTANDING_REQUEST)
 {}
 
 DefaultBtInteractive::~DefaultBtInteractive() {}
 
 void DefaultBtInteractive::initiateHandshake() {
   SharedHandle<BtMessage> message =
-    messageFactory->createHandshakeMessage(_btContext->getInfoHash(),
-					   _btContext->getPeerId());
+    messageFactory->createHandshakeMessage
+    (bittorrent::getInfoHash(_downloadContext), bittorrent::getStaticPeerId());
   dispatcher->addMessageToQueue(message);
   dispatcher->sendMessages();
 }
@@ -100,7 +106,7 @@ BtMessageHandle DefaultBtInteractive::receiveHandshake(bool quickReply) {
   if(message.isNull()) {
     return SharedHandle<BtMessage>();
   }
-  if(memcmp(message->getPeerId(), _btContext->getPeerId(),
+  if(memcmp(message->getPeerId(), bittorrent::getStaticPeerId(),
 	    PEER_ID_LENGTH) == 0) {
     throw DL_ABORT_EX
       (StringFormat
@@ -143,11 +149,15 @@ void DefaultBtInteractive::doPostHandshakeProcessing() {
   if(peer->isExtendedMessagingEnabled()) {
     addHandshakeExtendedMessageToQueue();
   }
-  addBitfieldMessageToQueue();
+  if(!_metadataGetMode) {
+    addBitfieldMessageToQueue();
+  }
   if(peer->isDHTEnabled() && _dhtEnabled) {
     addPortMessageToQueue();
   }
-  addAllowedFastMessageToQueue();  
+  if(!_metadataGetMode) {
+    addAllowedFastMessageToQueue();
+  }
   sendPendingMessage();
 }
 
@@ -159,12 +169,15 @@ void DefaultBtInteractive::addPortMessageToQueue()
 
 void DefaultBtInteractive::addHandshakeExtendedMessageToQueue()
 {
-  static const std::string CLIENT_ARIA2("aria2");
+  static const std::string CLIENT_ARIA2("aria2/"PACKAGE_VERSION);
   HandshakeExtensionMessageHandle m(new HandshakeExtensionMessage());
   m->setClientVersion(CLIENT_ARIA2);
   m->setTCPPort(_btRuntime->getListenPort());
   m->setExtensions(_extensionMessageRegistry->getExtensions());
-  
+  const BDE& attrs = _downloadContext->getAttribute(bittorrent::BITTORRENT);
+  if(attrs.containsKey(bittorrent::METADATA)) {
+    m->setMetadataSize(attrs[bittorrent::METADATA_SIZE].i());
+  }
   SharedHandle<BtMessage> msg = messageFactory->createBtExtendedMessage(m);
   dispatcher->addMessageToQueue(msg);
 }
@@ -187,9 +200,12 @@ void DefaultBtInteractive::addBitfieldMessageToQueue() {
 
 void DefaultBtInteractive::addAllowedFastMessageToQueue() {
   if(peer->isFastExtensionEnabled()) {
-    std::deque<size_t> fastSet;
-    _btContext->computeFastSet(fastSet, peer->ipaddr, allowedFastSetSize);
-    for(std::deque<size_t>::const_iterator itr = fastSet.begin();
+    std::vector<size_t> fastSet;
+    bittorrent::computeFastSet(fastSet, peer->ipaddr,
+			       _downloadContext->getNumPieces(),
+			       bittorrent::getInfoHash(_downloadContext),
+			       allowedFastSetSize);
+    for(std::vector<size_t>::const_iterator itr = fastSet.begin();
 	itr != fastSet.end(); ++itr) {
       dispatcher->addMessageToQueue
 	(messageFactory->createAllowedFastMessage(*itr));
@@ -236,10 +252,11 @@ void DefaultBtInteractive::sendKeepAlive() {
 }
 
 size_t DefaultBtInteractive::receiveMessages() {
+  size_t countOldOutstandingRequest = dispatcher->countOutstandingRequest();
   size_t msgcount = 0;
   for(int i = 0; i < 50; ++i) {
     if(_requestGroupMan->doesOverallDownloadSpeedExceed() ||
-       _btContext->getOwnerRequestGroup()->doesDownloadSpeedExceed()) {
+       _downloadContext->getOwnerRequestGroup()->doesDownloadSpeedExceed()) {
       break;
     }
     BtMessageHandle message = btMessageReceiver->receiveMessage();
@@ -274,6 +291,11 @@ size_t DefaultBtInteractive::receiveMessages() {
       break;
     }
   }
+  if(countOldOutstandingRequest > 0 &&
+     dispatcher->countOutstandingRequest() == 0){
+    _maxOutstandingRequest = std::min((size_t)UB_MAX_OUTSTANDING_REQUEST,
+				      _maxOutstandingRequest*2);
+  }
   return msgcount;
 }
 
@@ -333,23 +355,14 @@ void DefaultBtInteractive::fillPiece(size_t maxMissingBlock) {
 }
 
 void DefaultBtInteractive::addRequests() {
-  size_t MAX_PENDING_REQUEST;
-  if(peer->getLatency() < 500) {
-    MAX_PENDING_REQUEST = 24;
-  } else if(peer->getLatency() < 1500) {
-    MAX_PENDING_REQUEST = 12;
-  } else {
-    MAX_PENDING_REQUEST = 6;
-  }
-  fillPiece(MAX_PENDING_REQUEST);
-
+  fillPiece(_maxOutstandingRequest);
   size_t reqNumToCreate =
-    MAX_PENDING_REQUEST <= dispatcher->countOutstandingRequest() ?
-    0 : MAX_PENDING_REQUEST-dispatcher->countOutstandingRequest();
+    _maxOutstandingRequest <= dispatcher->countOutstandingRequest() ?
+    0 : _maxOutstandingRequest-dispatcher->countOutstandingRequest();
   if(reqNumToCreate > 0) {
     BtMessages requests;
     if(_pieceStorage->isEndGame()) {
-      btRequestFactory->createRequestMessagesOnEndGame(requests, reqNumToCreate);
+      btRequestFactory->createRequestMessagesOnEndGame(requests,reqNumToCreate);
     } else {
       btRequestFactory->createRequestMessages(requests, reqNumToCreate);
     }
@@ -359,6 +372,16 @@ void DefaultBtInteractive::addRequests() {
 
 void DefaultBtInteractive::cancelAllPiece() {
   btRequestFactory->removeAllTargetPiece();
+  if(_metadataGetMode && _downloadContext->getTotalLength() > 0) {
+    std::vector<size_t> metadataRequests =
+      _utMetadataRequestTracker->getAllTrackedIndex();
+    for(std::vector<size_t>::const_iterator i = metadataRequests.begin();
+	i != metadataRequests.end(); ++i) {
+      logger->debug("Cancel metadata: piece=%lu",
+		    static_cast<unsigned long>(*i));
+      _pieceStorage->cancelPiece(_pieceStorage->getPiece(*i));
+    }
+  }
 }
 
 void DefaultBtInteractive::sendPendingMessage() {
@@ -395,7 +418,7 @@ void DefaultBtInteractive::checkActiveInteraction()
   // mutual download progress are completely waste of resources, those peers
   // are disconnected in a certain time period.
   {
-    const time_t interval = 2*60;
+    const time_t interval = 60;
     if(inactiveCheckPoint.elapsed(interval)) {
       throw DL_ABORT_EX
 	(StringFormat(EX_DROP_INACTIVE_CONNECTION, interval).str());
@@ -433,29 +456,62 @@ void DefaultBtInteractive::addPeerExchangeMessage()
 }
 
 void DefaultBtInteractive::doInteractionProcessing() {
-  checkActiveInteraction();
+  if(_metadataGetMode) {
+    sendKeepAlive();
+    _numReceivedMessage = receiveMessages();
+    // PieceStorage is re-initialized with metadata_size in
+    // HandshakeExtensionMessage::doReceivedAction().
+    _pieceStorage =
+      _downloadContext->getOwnerRequestGroup()->getPieceStorage();
 
-  decideChoking();
+    if(peer->getExtensionMessageID("ut_metadata") &&
+       _downloadContext->getTotalLength() > 0) {
+      size_t num = _utMetadataRequestTracker->avail();
+      if(num > 0) {
+	std::deque<SharedHandle<BtMessage> > requests;
+	_utMetadataRequestFactory->create(requests, num, _pieceStorage);
+	dispatcher->addMessageToQueue(requests);
+      }
+      if(_perSecCheckPoint.elapsed(1)) {
+	_perSecCheckPoint.reset();
+	// Drop timeout request after queuing message to give a chance
+	// to other connection to request piece.
+	std::vector<size_t> indexes =
+	  _utMetadataRequestTracker->removeTimeoutEntry();
+	for(std::vector<size_t>::const_iterator i = indexes.begin();
+	    i != indexes.end(); ++i) {
+	  _pieceStorage->cancelPiece(_pieceStorage->getPiece(*i));
+	}
+      }
+      if(_pieceStorage->downloadFinished()) {
+	_downloadContext->getOwnerRequestGroup()->setForceHaltRequested
+	  (true, RequestGroup::NONE);
+      }
+    }
+  } else {
+    checkActiveInteraction();
 
-  detectMessageFlooding();
+    decideChoking();
 
-  if(_perSecCheckPoint.elapsed(1)) {
-    _perSecCheckPoint.reset();
-    dispatcher->checkRequestSlotAndDoNecessaryThing();
-  }
-  checkHave();
+    detectMessageFlooding();
 
-  sendKeepAlive();
+    if(_perSecCheckPoint.elapsed(1)) {
+      _perSecCheckPoint.reset();
+      dispatcher->checkRequestSlotAndDoNecessaryThing();
+    }
+    checkHave();
 
-  _numReceivedMessage = receiveMessages();
+    sendKeepAlive();
+
+    _numReceivedMessage = receiveMessages();
   
-  btRequestFactory->removeCompletedPiece();
+    btRequestFactory->removeCompletedPiece();
 
-  decideInterest();
-  if(!_pieceStorage->downloadFinished()) {
-    addRequests();
+    decideInterest();
+    if(!_pieceStorage->downloadFinished()) {
+      addRequests();
+    }
   }
-
   if(peer->getExtensionMessageID("ut_pex") && _utPexEnabled) {
     addPeerExchangeMessage();
   }
@@ -485,7 +541,11 @@ size_t DefaultBtInteractive::countReceivedMessageInIteration() const
 
 size_t DefaultBtInteractive::countOutstandingRequest()
 {
-  return dispatcher->countOutstandingRequest();
+  if(_metadataGetMode) {
+    return _utMetadataRequestTracker->count();
+  } else {
+    return dispatcher->countOutstandingRequest();
+  }
 }
 
 void DefaultBtInteractive::setBtRuntime
