@@ -35,10 +35,16 @@
 #include "FileAllocationCommand.h"
 #include "FileAllocationMan.h"
 #include "FileAllocationEntry.h"
+#include "DownloadEngine.h"
+#include "RequestGroup.h"
+#include "Logger.h"
 #include "message.h"
-#include "DownloadCommand.h"
 #include "prefs.h"
-#include "Util.h"
+#include "util.h"
+#include "DownloadEngine.h"
+#include "DownloadContext.h"
+
+namespace aria2 {
 
 FileAllocationCommand::FileAllocationCommand(int cuid, RequestGroup* requestGroup, DownloadEngine* e, const FileAllocationEntryHandle& fileAllocationEntry):
   RealtimeCommand(cuid, requestGroup, e),
@@ -55,11 +61,13 @@ bool FileAllocationCommand::executeInternal()
   if(_fileAllocationEntry->finished()) {
     logger->debug(MSG_ALLOCATION_COMPLETED,
 		  _timer.difference(),
-		  Util::llitos(_requestGroup->getTotalLength(), true).c_str());
-    _e->_fileAllocationMan->markCurrentFileAllocationEntryDone();
-    
-    _e->addCommand(_fileAllocationEntry->prepareForNextAction(_e));
+		  util::itos(_requestGroup->getTotalLength(), true).c_str());
+    _e->_fileAllocationMan->dropPickedEntry();
     
+    std::deque<Command*> commands;
+    _fileAllocationEntry->prepareForNextAction(commands, _e);
+    _e->addCommand(commands);
+    _e->setNoWait(true);
     return true;
   } else {
     _e->commands.push_back(this);
@@ -67,10 +75,13 @@ bool FileAllocationCommand::executeInternal()
   }
 }
 
-bool FileAllocationCommand::handleException(Exception* e)
+bool FileAllocationCommand::handleException(Exception& e)
 {
-  _e->_fileAllocationMan->markCurrentFileAllocationEntryDone();
+  _e->_fileAllocationMan->dropPickedEntry();
   logger->error(MSG_FILE_ALLOCATION_FAILURE, e, cuid);
-  logger->error(MSG_DOWNLOAD_NOT_COMPLETE, cuid, _requestGroup->getFilePath().c_str());
+  logger->error(MSG_DOWNLOAD_NOT_COMPLETE, cuid,
+		_requestGroup->getDownloadContext()->getBasePath().c_str());
   return true;
 }
+
+} // namespace aria2
