@@ -37,17 +37,29 @@
 
 #include "DownloadCommand.h"
 
+namespace aria2 {
+
+class FtpConnection;
+
 class FtpDownloadCommand : public DownloadCommand {
 private:
-  SocketHandle ctrlSocket;
+  SharedHandle<FtpConnection> _ftpConnection;
+
+  SharedHandle<SocketCore> ctrlSocket;
+protected:
+  virtual bool prepareForNextSegment();
 public:
   FtpDownloadCommand(int cuid,
-		     const RequestHandle req,
+		     const SharedHandle<Request>& req,
+		     const SharedHandle<FileEntry>& fileEntry,
 		     RequestGroup* requestGroup,
+		     const SharedHandle<FtpConnection>& ftpConnection,
 		     DownloadEngine* e,
-		     const SocketHandle& dataSocket,
-		     const SocketHandle& ctrlSocket);
+		     const SharedHandle<SocketCore>& dataSocket,
+		     const SharedHandle<SocketCore>& ctrlSocket);
   virtual ~FtpDownloadCommand();
 };
 
+} // namespace aria2
+
 #endif // _D_FTP_DOWNLOAD_COMMAND_H_
