@@ -34,17 +34,31 @@
 /* copyright --> */
 #include "FtpTunnelRequestCommand.h"
 #include "FtpTunnelResponseCommand.h"
+#include "Request.h"
+#include "Socket.h"
+#include "DownloadContext.h"
 
-FtpTunnelRequestCommand::FtpTunnelRequestCommand(int cuid,
-						 const RequestHandle& req,
-						 RequestGroup* requestGroup,
-						 DownloadEngine* e,
-						 const SocketHandle& s)
-  :AbstractProxyRequestCommand(cuid, req, requestGroup, e, s) {}
+namespace aria2 {
+
+FtpTunnelRequestCommand::FtpTunnelRequestCommand
+(int cuid,
+ const RequestHandle& req,
+ const SharedHandle<FileEntry>& fileEntry,
+ RequestGroup* requestGroup,
+ DownloadEngine* e,
+ const SharedHandle<Request>& proxyRequest,
+ const SocketHandle& s)
+  :
+  AbstractProxyRequestCommand(cuid, req, fileEntry, requestGroup, e,
+			      proxyRequest, s)
+{}
 
 FtpTunnelRequestCommand::~FtpTunnelRequestCommand() {}
 
 Command* FtpTunnelRequestCommand::getNextCommand()
 {
-  return new FtpTunnelResponseCommand(cuid, req, _requestGroup, httpConnection, e, socket);
+  return new FtpTunnelResponseCommand
+    (cuid, req, _fileEntry, _requestGroup, httpConnection, e, socket);
 }
+
+} // namespace aria2
