@@ -36,12 +36,13 @@
 
 #include <cassert>
 #include <numeric>
+#include <vector>
 
 #include "Segment.h"
 #include "Range.h"
 #include "CookieStorage.h"
 #include "Option.h"
-#include "Util.h"
+#include "util.h"
 #include "Base64.h"
 #include "prefs.h"
 #include "AuthConfigFactory.h"
@@ -53,8 +54,7 @@ namespace aria2 {
 
 const std::string HttpRequest::USER_AGENT("aria2");
 
-HttpRequest::HttpRequest():entityLength(0),
-			   _contentEncodingEnabled(true),
+HttpRequest::HttpRequest():_contentEncodingEnabled(true),
 			   userAgent(USER_AGENT)
 {}
 
@@ -73,7 +73,7 @@ off_t HttpRequest::getStartByte() const
   if(segment.isNull()) {
     return 0;
   } else {
-    return segment->getPositionToWrite();
+    return _fileEntry->gtoloff(segment->getPositionToWrite());
   }
 }
 
@@ -83,7 +83,8 @@ off_t HttpRequest::getEndByte() const
     return 0;
   } else {
     if(request->isPipeliningEnabled()) {
-      return segment->getPosition()+segment->getLength()-1;
+      off_t endByte = _fileEntry->gtoloff(segment->getPosition()+segment->getLength()-1);
+      return std::min(endByte, static_cast<off_t>(_fileEntry->getLength()-1));
     } else {
       return 0;
     }
@@ -96,7 +97,8 @@ RangeHandle HttpRequest::getRange() const
   if(segment.isNull()) {
     return SharedHandle<Range>(new Range());
   } else {
-    return SharedHandle<Range>(new Range(getStartByte(), getEndByte(), entityLength));
+    return SharedHandle<Range>(new Range(getStartByte(), getEndByte(),
+					 _fileEntry->getLength()));
   }
 }
 
@@ -107,27 +109,27 @@ bool HttpRequest::isRangeSatisfied(const RangeHandle& range) const
   }
   if((getStartByte() == range->getStartByte()) &&
      ((getEndByte() == 0) ||
-      ((getEndByte() > 0) && (getEndByte() == range->getEndByte()))) &&
-     ((entityLength == 0) ||
-      ((entityLength > 0) && (entityLength == range->getEntityLength())))) {
+      (getEndByte() == range->getEndByte())) &&
+     ((_fileEntry->getLength() == 0) ||
+      (_fileEntry->getLength() == range->getEntityLength()))) {
     return true;
   } else {
     return false;
   }  
 }
 
-std::string HttpRequest::getHostText(const std::string& host, uint16_t port) const
+static std::string getHostText(const std::string& host, uint16_t port)
 {
   std::string hosttext = host;
   if(!(port == 80 || port == 443)) {
-    strappend(hosttext, ":", Util::uitos(port));
+    strappend(hosttext, ":", util::uitos(port));
   }
   return hosttext;
 }
 
 std::string HttpRequest::createRequest()
 {
-  _authConfig = _authConfigFactory->createAuthConfig(request);
+  _authConfig = _authConfigFactory->createAuthConfig(request, _option);
   std::string requestLine = request->getMethod();
   requestLine += " ";
   if(!_proxyRequest.isNull()) {
@@ -136,7 +138,7 @@ std::string HttpRequest::createRequest()
       // Insert user into URI, like ftp://USER@host/
       std::string uri = getCurrentURI();
       assert(uri.size() >= 6);
-      uri.insert(6, Util::urlencode(_authConfig->getUser())+"@");
+      uri.insert(6, util::urlencode(_authConfig->getUser())+"@");
       requestLine += uri;
     } else {
       requestLine += getCurrentURI();
@@ -171,7 +173,7 @@ std::string HttpRequest::createRequest()
     }
   }
 
-  strappend(requestLine, "Host: ", getHostText(getHost(), getPort()), "\r\n");
+  strappend(requestLine, "Host: ", getHostText(getURIHost(), getPort()), "\r\n");
   requestLine += "Pragma: no-cache\r\n";
   requestLine += "Cache-Control: no-cache\r\n";
 
@@ -181,10 +183,10 @@ std::string HttpRequest::createRequest()
   if(!segment.isNull() && segment->getLength() > 0 && 
      (request->isPipeliningEnabled() || getStartByte() > 0)) {
     requestLine += "Range: bytes=";
-    requestLine += Util::itos(getStartByte());
+    requestLine += util::itos(getStartByte());
     requestLine += "-";
     if(request->isPipeliningEnabled()) {
-      requestLine += Util::itos(getEndByte());
+      requestLine += util::itos(getEndByte());
     }
     requestLine += "\r\n";
   }
@@ -234,8 +236,8 @@ std::string HttpRequest::createRequest()
 std::string HttpRequest::createProxyRequest() const
 {
   assert(!_proxyRequest.isNull());
-  std::string hostport = getHost();
-  strappend(hostport, ":", Util::uitos(getPort()));
+  std::string hostport = getURIHost();
+  strappend(hostport, ":", util::uitos(getPort()));
 
   std::string requestLine = "CONNECT ";
   strappend(requestLine, hostport, " HTTP/1.1\r\n");
@@ -275,8 +277,8 @@ void HttpRequest::disableContentEncoding()
 
 void HttpRequest::addHeader(const std::string& headersString)
 {
-  std::deque<std::string> headers;
-  Util::slice(headers, headersString, '\n', true);
+  std::vector<std::string> headers;
+  util::split(headersString, std::back_inserter(headers), "\n", true);
   _headers.insert(_headers.end(), headers.begin(), headers.end());
 }
 
@@ -292,9 +294,10 @@ void HttpRequest::setCookieStorage
 }
 
 void HttpRequest::setAuthConfigFactory
-(const SharedHandle<AuthConfigFactory>& factory)
+(const SharedHandle<AuthConfigFactory>& factory, const Option* option)
 {
   _authConfigFactory = factory;
+  _option = option;
 }
 
 void HttpRequest::setProxyRequest(const SharedHandle<Request>& proxyRequest)
