@@ -44,7 +44,7 @@
 #include "Logger.h"
 #include "HttpRequest.h"
 #include "Segment.h"
-#include "Util.h"
+#include "util.h"
 #include "StringFormat.h"
 #include "DlAbortEx.h"
 #include "HttpHeader.h"
@@ -53,18 +53,20 @@
 #include "CookieStorage.h"
 #include "AuthConfigFactory.h"
 #include "AuthConfig.h"
+#include "DownloadContext.h"
 
 namespace aria2 {
 
 HttpSkipResponseCommand::HttpSkipResponseCommand
 (int cuid,
  const SharedHandle<Request>& req,
+ const SharedHandle<FileEntry>& fileEntry,
  RequestGroup* requestGroup,
  const SharedHandle<HttpConnection>& httpConnection,
  const SharedHandle<HttpResponse>& httpResponse,
  DownloadEngine* e,
  const SharedHandle<SocketCore>& s):
-  AbstractCommand(cuid, req, requestGroup, e, s),
+  AbstractCommand(cuid, req, fileEntry, requestGroup, e, s),
   _httpConnection(httpConnection),
   _httpResponse(httpResponse),
   _totalLength(_httpResponse->getEntityLength()),
@@ -155,23 +157,21 @@ bool HttpSkipResponseCommand::processResponse()
     }
     _httpResponse->processRedirect();
     return prepareForRetry(0);
-  } else if(_httpResponse->hasRetryAfter()) {
-    return prepareForRetry(_httpResponse->getRetryAfter());
   } else if(_httpResponse->getResponseStatus() >= HttpHeader::S400) {
     if(_httpResponse->getResponseStatus() == HttpHeader::S401) {
       if(getOption()->getAsBool(PREF_HTTP_AUTH_CHALLENGE) &&
 	 !_httpResponse->getHttpRequest()->authenticationUsed() &&
 	 e->getAuthConfigFactory()->activateBasicCred
-	 (req->getHost(), req->getDir())) {
+	 (req->getHost(), req->getDir(), getOption().get())) {
 	return prepareForRetry(0);
       } else {
 	throw DL_ABORT_EX(EX_AUTH_FAILED);
       }
     }else if(_httpResponse->getResponseStatus() == HttpHeader::S404) {
       throw DL_ABORT_EX2(MSG_RESOURCE_NOT_FOUND,
-			 DownloadResult::RESOURCE_NOT_FOUND);
+			 downloadresultcode::RESOURCE_NOT_FOUND);
     } else {
-      throw DL_ABORT_EX(StringFormat(EX_BAD_STATUS, Util::parseUInt(_httpResponse->getResponseStatus())).str());
+      throw DL_ABORT_EX(StringFormat(EX_BAD_STATUS, util::parseUInt(_httpResponse->getResponseStatus())).str());
     }
   } else {
     return prepareForRetry(0);
