@@ -38,7 +38,7 @@
 
 #include "MetalinkResource.h"
 #include "FileEntry.h"
-#include "Util.h"
+#include "util.h"
 #include "a2functional.h"
 #ifdef ENABLE_MESSAGE_DIGEST
 # include "Checksum.h"
@@ -64,7 +64,7 @@ public:
   AddLocationPreference(const std::deque<std::string>& locations, int preferenceToAdd):
     _locations(locations), _preferenceToAdd(preferenceToAdd)
   {
-    std::transform(_locations.begin(), _locations.end(), _locations.begin(), Util::toUpper);
+    std::transform(_locations.begin(), _locations.end(), _locations.begin(), util::toUpper);
     std::sort(_locations.begin(), _locations.end());
   }
 
@@ -181,19 +181,9 @@ void MetalinkEntry::toFileEntry
 		 mem_fun_sh(&MetalinkEntry::getFile));
 }
 
-SharedHandle<FileEntry> MetalinkEntry::getFile() const
-{
-  return file;
-}
-
 void MetalinkEntry::setSignature(const SharedHandle<Signature>& signature)
 {
   _signature = signature;
 }
 
-SharedHandle<Signature> MetalinkEntry::getSignature() const
-{
-  return _signature;
-}
-
 } // namespace aria2
