@@ -38,9 +38,10 @@
 #include "array_fun.h"
 #include "usage_text.h"
 #include "A2STR.h"
-#include "Util.h"
+#include "util.h"
 #include "help_tags.h"
 #include "StringFormat.h"
+#include "File.h"
 
 namespace aria2 {
 
@@ -54,6 +55,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
 				    TEXT_ALLOW_OVERWRITE,
 				    V_FALSE));
     op->addTag(TAG_ADVANCED);
+    op->addTag(TAG_FILE);
     handlers.push_back(op);
   }
   {
@@ -64,20 +66,25 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   } 
+#ifdef ENABLE_ASYNC_DNS
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_ASYNC_DNS,
 				    TEXT_ASYNC_DNS,
-				    V_TRUE));
+				    V_TRUE,
+				    OptionHandler::OPT_ARG));
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
+#endif // ENABLE_ASYNC_DNS
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_AUTO_FILE_RENAMING,
 				    TEXT_AUTO_FILE_RENAMING,
-				    V_TRUE));
+				    V_TRUE,
+				    OptionHandler::OPT_ARG));
     op->addTag(TAG_ADVANCED);
+    op->addTag(TAG_FILE);
     handlers.push_back(op);
   }
   {
@@ -89,21 +96,26 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
+#ifdef ENABLE_MESSAGE_DIGEST
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_CHECK_INTEGRITY,
 				    TEXT_CHECK_INTEGRITY,
-				    V_FALSE));
+				    V_FALSE,
+				    OptionHandler::OPT_ARG,
+				    'V'));
     op->addTag(TAG_BASIC);
     op->addTag(TAG_BITTORRENT);
     op->addTag(TAG_METALINK);
+    op->addTag(TAG_FILE);
     handlers.push_back(op);
   }
+#endif // ENABLE_MESSAGE_DIGEST
   {
     SharedHandle<OptionHandler> op(new DefaultOptionHandler
 				   (PREF_CONF_PATH,
 				    TEXT_CONF_PATH,
-				    Util::getHomeDir()+"/.aria2/aria2.conf"));
+				    util::getHomeDir()+"/.aria2/aria2.conf"));
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
@@ -111,7 +123,9 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_CONTINUE,
 				    TEXT_CONTINUE,
-				    V_FALSE)); // TODO ommit?
+				    V_FALSE, // TODO ommit?
+				    OptionHandler::NO_ARG,
+				    'c'));
     op->addTag(TAG_BASIC);
     op->addTag(TAG_FTP);
     op->addTag(TAG_HTTP);
@@ -121,7 +135,9 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_DAEMON,
 				    TEXT_DAEMON,
-				    V_FALSE)); // TODO ommit?
+				    V_FALSE,
+				    OptionHandler::NO_ARG,
+				    'D')); // TODO ommit?
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
@@ -129,24 +145,72 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new DefaultOptionHandler
 				   (PREF_DIR,
 				    TEXT_DIR,
-				    "."));
+				    File::getCurrentDir(),
+				    A2STR::NIL,
+				    OptionHandler::REQ_ARG,
+				    'd'));
     op->addTag(TAG_BASIC);
-    handlers.push_back(op);
-  }
-  {
-    SharedHandle<OptionHandler> op(new NumberOptionHandler
-				   (PREF_DNS_TIMEOUT,
-				    NO_DESCRIPTION,
-				    "30",
-				    1, 60,
-				    true));
+    op->addTag(TAG_FILE);
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new BooleanOptionHandler
+				   (PREF_DISABLE_IPV6,
+				    TEXT_DISABLE_IPV6,
+				    V_FALSE,
+				    OptionHandler::OPT_ARG));
+    op->addTag(TAG_ADVANCED);
+    handlers.push_back(op);
+  }
+  {
+    SharedHandle<NumberOptionHandler> op(new NumberOptionHandler
+					 (PREF_DNS_TIMEOUT,
+					  NO_DESCRIPTION,
+					  "30",
+					  1, 60));
+    op->hide();
+    handlers.push_back(op);
+  }
+#ifdef ENABLE_DIRECT_IO
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_ENABLE_DIRECT_IO,
 				    TEXT_ENABLE_DIRECT_IO,
-				    V_TRUE));
+				    V_TRUE,
+				    OptionHandler::OPT_ARG));
+    op->addTag(TAG_ADVANCED);
+    op->addTag(TAG_FILE);
+    handlers.push_back(op);
+  }
+#endif // ENABLE_DIRECT_IO
+#ifdef ENABLE_XML_RPC
+  {
+    SharedHandle<OptionHandler> op(new BooleanOptionHandler
+				   (PREF_ENABLE_XML_RPC,
+				    TEXT_ENABLE_XML_RPC,
+				    V_FALSE,
+				    OptionHandler::OPT_ARG));
+    op->addTag(TAG_XML_RPC);
+    handlers.push_back(op);
+  }
+#endif // ENABLE_XML_RPC
+  {
+    std::string params[] = {
+#ifdef HAVE_EPOLL
+      V_EPOLL,
+#endif // HAVE_EPOLL
+      V_SELECT
+    };
+    SharedHandle<OptionHandler> op(new ParameterOptionHandler
+				   (PREF_EVENT_POLL,
+				    TEXT_EVENT_POLL,
+#ifdef HAVE_EPOLL
+				    V_EPOLL,
+#else // !HAVE_EPOLL
+				    V_SELECT,
+#endif // !HAVE_EPOLL
+				    std::deque<std::string>
+				    (&params[0],&params[arrayLength(params)])));
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
@@ -155,15 +219,22 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
 				   (PREF_FILE_ALLOCATION,
 				    TEXT_FILE_ALLOCATION,
 				    V_PREALLOC,
-				    V_NONE, V_PREALLOC));
+				    V_NONE, V_PREALLOC,
+#ifdef HAVE_POSIX_FALLOCATE
+				    V_FALLOC,
+#endif // HAVE_POSIX_FALLOCATE
+				    'a'));
     op->addTag(TAG_BASIC);
+    op->addTag(TAG_FILE);
     handlers.push_back(op);
   }
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_FORCE_SEQUENTIAL,
 				    TEXT_FORCE_SEQUENTIAL,
-				    V_FALSE));
+				    V_FALSE,
+				    OptionHandler::OPT_ARG,
+				    'Z'));
     op->addTag(TAG_BASIC);
     handlers.push_back(op);
   }
@@ -172,16 +243,30 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
 				   (PREF_INPUT_FILE,
 				    TEXT_INPUT_FILE,
 				    NO_DEFAULT_VALUE,
-				    "FILENAME,-"));
+				    "FILENAME,-",
+				    OptionHandler::REQ_ARG,
+				    'i'));
     op->addTag(TAG_BASIC);
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new DefaultOptionHandler
+				   (PREF_INTERFACE,
+				    TEXT_INTERFACE,
+				    NO_DEFAULT_VALUE,
+				    "interface, IP address, hostname",
+				    OptionHandler::REQ_ARG));
+    op->addTag(TAG_ADVANCED);
+    handlers.push_back(op);
+  }
   {
     SharedHandle<OptionHandler> op(new DefaultOptionHandler
 				   (PREF_LOG,
 				    TEXT_LOG,
 				    NO_DEFAULT_VALUE,
-				    "FILENAME,-"));
+				    "FILENAME,-",
+				    OptionHandler::REQ_ARG,
+				    'l'));
     op->addTag(TAG_BASIC);
     handlers.push_back(op);
   }
@@ -202,7 +287,8 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
 				   (PREF_MAX_CONCURRENT_DOWNLOADS,
 				    TEXT_MAX_CONCURRENT_DOWNLOADS,
 				    "5",
-				    1, 45));
+				    1, -1,
+				    'j'));
     op->addTag(TAG_BASIC);
     handlers.push_back(op);
   }
@@ -217,11 +303,23 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new UnitNumberOptionHandler
+				   (PREF_MAX_OVERALL_DOWNLOAD_LIMIT,
+				    TEXT_MAX_OVERALL_DOWNLOAD_LIMIT,
+				    "0",
+				    0));
+    op->addTag(TAG_BITTORRENT);
+    op->addTag(TAG_FTP);
+    op->addTag(TAG_HTTP);
+    handlers.push_back(op);
+  }
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_NO_CONF,
 				    TEXT_NO_CONF,
-				    V_FALSE));
+				    V_FALSE,
+				    OptionHandler::NO_ARG));
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
@@ -232,13 +330,56 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
 				    "5M",
 				    0));
     op->addTag(TAG_ADVANCED);
+    op->addTag(TAG_FILE);
+    handlers.push_back(op);
+  }
+  {
+    SharedHandle<OptionHandler> op(new DefaultOptionHandler
+				   (PREF_ON_DOWNLOAD_COMPLETE,
+				    TEXT_ON_DOWNLOAD_COMPLETE,
+				    NO_DEFAULT_VALUE,
+				    "/path/to/command"));
+    op->addTag(TAG_ADVANCED);
+    op->addTag(TAG_HOOK);
+    handlers.push_back(op);
+  }
+  {
+    SharedHandle<OptionHandler> op(new DefaultOptionHandler
+				   (PREF_ON_DOWNLOAD_ERROR,
+				    TEXT_ON_DOWNLOAD_ERROR,
+				    NO_DEFAULT_VALUE,
+				    "/path/to/command"));
+    op->addTag(TAG_ADVANCED);
+    op->addTag(TAG_HOOK);
+    handlers.push_back(op);
+  }
+  {
+    SharedHandle<OptionHandler> op(new DefaultOptionHandler
+				   (PREF_ON_DOWNLOAD_START,
+				    TEXT_ON_DOWNLOAD_START,
+				    NO_DEFAULT_VALUE,
+				    "/path/to/command"));
+    op->addTag(TAG_ADVANCED);
+    op->addTag(TAG_HOOK);
+    handlers.push_back(op);
+  }
+  {
+    SharedHandle<OptionHandler> op(new DefaultOptionHandler
+				   (PREF_ON_DOWNLOAD_STOP,
+				    TEXT_ON_DOWNLOAD_STOP,
+				    NO_DEFAULT_VALUE,
+				    "/path/to/command"));
+    op->addTag(TAG_ADVANCED);
+    op->addTag(TAG_HOOK);
     handlers.push_back(op);
   }
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_PARAMETERIZED_URI,
 				    TEXT_PARAMETERIZED_URI,
-				    V_FALSE));  
+				    V_FALSE,
+				    OptionHandler::OPT_ARG,
+				    'P'));  
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
@@ -246,10 +387,13 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_QUIET,
 				    TEXT_QUIET,
-				    V_FALSE));
+				    V_FALSE,
+				    OptionHandler::OPT_ARG,
+				    'q'));
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
+#ifdef ENABLE_MESSAGE_DIGEST
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_REALTIME_CHUNK_CHECKSUM,
@@ -258,6 +402,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_METALINK);
     handlers.push_back(op);
   }
+#endif // ENABLE_MESSAGE_DIGEST
   {
     SharedHandle<OptionHandler> op(new NumberOptionHandler
 				   (PREF_STOP,
@@ -276,6 +421,49 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_ADVANCED);
     handlers.push_back(op);
   }
+#ifdef ENABLE_XML_RPC
+  {
+    SharedHandle<OptionHandler> op(new BooleanOptionHandler
+				   (PREF_XML_RPC_LISTEN_ALL,
+				    TEXT_XML_RPC_LISTEN_ALL,
+				    V_FALSE,
+				    OptionHandler::OPT_ARG));
+    op->addTag(TAG_XML_RPC);
+    handlers.push_back(op);
+  }
+  {
+    SharedHandle<OptionHandler> op(new NumberOptionHandler
+				   (PREF_XML_RPC_LISTEN_PORT,
+				    TEXT_XML_RPC_LISTEN_PORT,
+				    "6800",
+				    1024, UINT16_MAX));
+    op->addTag(TAG_XML_RPC);
+    handlers.push_back(op);
+  }
+  {
+    SharedHandle<OptionHandler> op(new UnitNumberOptionHandler
+				   (PREF_XML_RPC_MAX_REQUEST_SIZE,
+				    TEXT_XML_RPC_MAX_REQUEST_SIZE,
+				    "2M",
+				    0));
+    op->addTag(TAG_XML_RPC);
+    handlers.push_back(op);
+  }			    
+  {
+    SharedHandle<OptionHandler> op(new DefaultOptionHandler
+				   (PREF_XML_RPC_USER,
+				    TEXT_XML_RPC_USER));
+    op->addTag(TAG_XML_RPC);
+    handlers.push_back(op);
+  }
+  {
+    SharedHandle<OptionHandler> op(new DefaultOptionHandler
+				   (PREF_XML_RPC_PASSWD,
+				    TEXT_XML_RPC_PASSWD));
+    op->addTag(TAG_XML_RPC);
+    handlers.push_back(op);
+  }			    
+#endif // ENABLE_XML_RPC
   // HTTP/FTP options
   {
     SharedHandle<OptionHandler> op(new NumberOptionHandler
@@ -287,6 +475,16 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new BooleanOptionHandler
+				   (PREF_DRY_RUN,
+				    TEXT_DRY_RUN,
+				    V_FALSE,
+				    OptionHandler::OPT_ARG));
+    op->addTag(TAG_FTP);
+    op->addTag(TAG_HTTP);
+    handlers.push_back(op);
+  }
   {
     SharedHandle<OptionHandler> op(new UnitNumberOptionHandler
 				   (PREF_LOWEST_SPEED_LIMIT,
@@ -312,7 +510,19 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
 				   (PREF_MAX_TRIES,
 				    TEXT_MAX_TRIES,
 				    "5",
-				    0));
+				    0, -1,
+				    'm'));
+    op->addTag(TAG_FTP);
+    op->addTag(TAG_HTTP);
+    handlers.push_back(op);
+  }
+  {
+    SharedHandle<OptionHandler> op(new BooleanOptionHandler
+				   (PREF_NO_NETRC,
+				    TEXT_NO_NETRC,
+				    V_FALSE, // TODO ommit?
+				    OptionHandler::NO_ARG,
+				    'n'));
     op->addTag(TAG_FTP);
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
@@ -322,38 +532,41 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
 				   (PREF_OUT,
 				    TEXT_OUT,
 				    NO_DEFAULT_VALUE,
-				    "FILENAME"));
+				    "FILENAME",
+				    OptionHandler::REQ_ARG,
+				    'o'));
     op->addTag(TAG_BASIC);
     op->addTag(TAG_FTP);
     op->addTag(TAG_HTTP);
+    op->addTag(TAG_FILE);
     handlers.push_back(op);
   }
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_REMOTE_TIME,
 				    TEXT_REMOTE_TIME,
-				    V_FALSE));
+				    V_FALSE,
+				    OptionHandler::OPT_ARG,
+				    'R'));
     op->addTag(TAG_FTP);
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
   }
   {
-    SharedHandle<OptionHandler> op(new NumberOptionHandler
-				   (PREF_RETRY_WAIT,
-				    TEXT_RETRY_WAIT,
-				    "5",
-				    0, 60));
-    op->addTag(TAG_FTP);
-    op->addTag(TAG_HTTP);
-    handlers.push_back(op);
-  }
-  {
-    SharedHandle<OptionHandler> op(new UnitNumberOptionHandler
-				   (PREF_SEGMENT_SIZE,
+    SharedHandle<OptionHandler> op(new BooleanOptionHandler
+				   (PREF_REUSE_URI,
 				    NO_DESCRIPTION,
-				    "1M",
-				    1024, -1,
-				    true));
+				    V_TRUE));
+    op->hide();
+    handlers.push_back(op);
+  }
+  {
+    SharedHandle<UnitNumberOptionHandler> op(new UnitNumberOptionHandler
+					     (PREF_SEGMENT_SIZE,
+					      NO_DESCRIPTION,
+					      "1M",
+					      1024, -1));
+    op->hide();
     handlers.push_back(op);
   }
   {
@@ -391,19 +604,20 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
 				   (PREF_SPLIT,
 				    TEXT_SPLIT,
 				    "5",
-				    1, 16));
+				    1, -1,
+				    's'));
     op->addTag(TAG_BASIC);
     op->addTag(TAG_FTP);
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
   }
   {
-    SharedHandle<OptionHandler> op(new NumberOptionHandler
-				   (PREF_STARTUP_IDLE_TIME,
-				    NO_DESCRIPTION,
-				    "10",
-				    1, 60,
-				    true));
+    SharedHandle<NumberOptionHandler> op(new NumberOptionHandler
+					 (PREF_STARTUP_IDLE_TIME,
+					  NO_DESCRIPTION,
+					  "10",
+					  1, 60));
+    op->hide();
     handlers.push_back(op);
   }
   {
@@ -411,17 +625,18 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
 				   (PREF_TIMEOUT,
 				    TEXT_TIMEOUT,
 				    "60",
-				    1, 600));
+				    1, 600,
+				    't'));
     op->addTag(TAG_FTP);
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
   }
   {
-    const std::string params[] = { V_INORDER, V_FEEDBACK };
+    const std::string params[] = { V_INORDER, V_FEEDBACK, V_ADAPTIVE };
     SharedHandle<OptionHandler> op(new ParameterOptionHandler
 				   (PREF_URI_SELECTOR,
 				    TEXT_URI_SELECTOR,
-				    V_INORDER,
+				    V_FEEDBACK,
 				    std::deque<std::string>
 				    (&params[0], &params[arrayLength(params)])));
     op->addTag(TAG_FTP);
@@ -450,7 +665,8 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_CHECK_CERTIFICATE,
 				    TEXT_CHECK_CERTIFICATE,
-				    V_TRUE));
+				    V_TRUE,
+				    OptionHandler::OPT_ARG));
     op->addTag(TAG_HTTP);
     op->addTag(TAG_HTTPS);
     handlers.push_back(op);
@@ -459,7 +675,8 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_ENABLE_HTTP_KEEP_ALIVE,
 				    TEXT_ENABLE_HTTP_KEEP_ALIVE,
-				    V_TRUE));
+				    V_TRUE,
+				    OptionHandler::OPT_ARG));
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
   }
@@ -467,7 +684,8 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_ENABLE_HTTP_PIPELINING,
 				    TEXT_ENABLE_HTTP_PIPELINING,
-				    V_FALSE));
+				    V_FALSE,
+				    OptionHandler::OPT_ARG));
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
   }
@@ -481,11 +699,11 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     handlers.push_back(op);
   }
   {
-    SharedHandle<OptionHandler> op(new ParameterOptionHandler
-				   (PREF_HTTP_AUTH_SCHEME,
-				    TEXT_HTTP_AUTH_SCHEME,
-				    V_BASIC,
-				    V_BASIC));
+    SharedHandle<OptionHandler> op(new BooleanOptionHandler
+				   (PREF_HTTP_AUTH_CHALLENGE,
+				    TEXT_HTTP_AUTH_CHALLENGE,
+				    V_FALSE,
+				    OptionHandler::OPT_ARG));
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
   }
@@ -510,18 +728,19 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
 				   (PREF_LOAD_COOKIES,
 				    TEXT_LOAD_COOKIES,
 				    NO_DEFAULT_VALUE,
-				    "FILENAME"));
+				    "/path/to/file"));
     op->addTag(TAG_BASIC);
     op->addTag(TAG_HTTP);
+    op->addTag(TAG_COOKIE);
     handlers.push_back(op);
   }
   {
-    SharedHandle<OptionHandler> op(new NumberOptionHandler
-				   (PREF_MAX_HTTP_PIPELINING,
-				    NO_DESCRIPTION,
-				    "2",
-				    1, 8,
-				    true));
+    SharedHandle<NumberOptionHandler> op(new NumberOptionHandler
+					 (PREF_MAX_HTTP_PIPELINING,
+					  NO_DESCRIPTION,
+					  "2",
+					  1, 8));
+    op->hide();
     handlers.push_back(op);
   }
   {
@@ -541,9 +760,20 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
   }
   {
     SharedHandle<OptionHandler> op(new DefaultOptionHandler
+				   (PREF_SAVE_COOKIES,
+				    TEXT_SAVE_COOKIES,
+				    NO_DEFAULT_VALUE,
+				    "/path/to/file"));
+    op->addTag(TAG_HTTP);
+    op->addTag(TAG_COOKIE);
+    handlers.push_back(op);
+  }
+  {
+    SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_USE_HEAD,
 				    TEXT_USE_HEAD,
-				    V_TRUE));
+				    V_FALSE,
+				    OptionHandler::OPT_ARG));
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
   }
@@ -551,7 +781,10 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new DefaultOptionHandler
 				   (PREF_USER_AGENT,
 				    TEXT_USER_AGENT,
-				    "aria2"));
+				    "aria2/"PACKAGE_VERSION,
+				    A2STR::NIL,
+				    OptionHandler::REQ_ARG,
+				    'U'));
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
   }
@@ -568,7 +801,9 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_FTP_PASV,
 				    TEXT_FTP_PASV,
-				    V_TRUE));
+				    V_TRUE,
+				    OptionHandler::OPT_ARG,
+				    'p'));
     op->addTag(TAG_FTP);
     handlers.push_back(op);
   }
@@ -576,7 +811,8 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_FTP_REUSE_CONNECTION,
 				    TEXT_FTP_REUSE_CONNECTION,
-				    V_TRUE));
+				    V_TRUE,
+				    OptionHandler::OPT_ARG));
     op->addTag(TAG_FTP);
     handlers.push_back(op);
   }
@@ -598,21 +834,12 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     handlers.push_back(op);
   }
   {
-    SharedHandle<OptionHandler> op(new DefaultOptionHandler
-				   (PREF_NETRC_PATH,
-				    NO_DESCRIPTION,
-				    Util::getHomeDir()+"/.netrc",
-				    "/PATH/TO/NETRC",
-				    true));
-    handlers.push_back(op);
-  }
-  {
-    SharedHandle<OptionHandler> op(new BooleanOptionHandler
-				   (PREF_NO_NETRC,
-				    TEXT_NO_NETRC,
-				    V_FALSE)); // TODO ommit?
-    op->addTag(TAG_FTP);
-    op->addTag(TAG_HTTP);
+    SharedHandle<DefaultOptionHandler> op(new DefaultOptionHandler
+					  (PREF_NETRC_PATH,
+					   NO_DESCRIPTION,
+					   util::getHomeDir()+"/.netrc",
+					   "/PATH/TO/NETRC"));
+    op->hide();
     handlers.push_back(op);
   }
   // Proxy options
@@ -630,6 +857,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
 				    TEXT_HTTPS_PROXY,
 				    NO_DEFAULT_VALUE));
     op->addTag(TAG_HTTP);
+    op->addTag(TAG_HTTPS);
     handlers.push_back(op);
   }
   {
@@ -647,6 +875,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
 				    NO_DEFAULT_VALUE));
     op->addTag(TAG_FTP);
     op->addTag(TAG_HTTP);
+    op->addTag(TAG_HTTPS);
     handlers.push_back(op);
   }
   {
@@ -655,19 +884,21 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
 				    TEXT_NO_PROXY));
     op->addTag(TAG_FTP);
     op->addTag(TAG_HTTP);
+    op->addTag(TAG_HTTPS);
     handlers.push_back(op);
   }
   {
     SharedHandle<OptionHandler> op(new ParameterOptionHandler
 				   (PREF_PROXY_METHOD,
 				    TEXT_PROXY_METHOD,
-				    V_TUNNEL,
+				    V_GET,
 				    V_GET, V_TUNNEL));
     op->addTag(TAG_FTP);
     op->addTag(TAG_HTTP);
     handlers.push_back(op);
   }
   // BitTorrent/Metalink Options
+#if defined ENABLE_BITTORRENT || defined ENABLE_METALINK
   {
     SharedHandle<OptionHandler> op(new IntegerRangeOptionHandler
 				   (PREF_SELECT_FILE,
@@ -682,20 +913,42 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_SHOW_FILES,
 				    TEXT_SHOW_FILES,
-				    V_FALSE)); // TODO ommit?
+				    V_FALSE, // TODO ommit?
+				    OptionHandler::NO_ARG,
+				    'S'));
     op->addTag(TAG_BASIC);
     op->addTag(TAG_BITTORRENT);
     op->addTag(TAG_METALINK);
     handlers.push_back(op);
   }
+#endif // ENABLE_BITTORRENT || ENABLE_METALINK
   // BitTorrent Specific Options
+#ifdef ENABLE_BITTORRENT
   {
-    SharedHandle<OptionHandler> op(new NumberOptionHandler
-				   (PREF_BT_KEEP_ALIVE_INTERVAL,
-				    NO_DESCRIPTION,
-				    "120",
-				    1, 120,
-				    true));
+    SharedHandle<OptionHandler> op(new DefaultOptionHandler
+				   (PREF_BT_EXTERNAL_IP,
+				    TEXT_BT_EXTERNAL_IP,
+				    NO_DEFAULT_VALUE,
+				    "a numeric IP address"));
+    op->addTag(TAG_BITTORRENT);
+    handlers.push_back(op);
+  }
+  {
+    SharedHandle<NumberOptionHandler> op(new NumberOptionHandler
+					 (PREF_BT_KEEP_ALIVE_INTERVAL,
+					  NO_DESCRIPTION,
+					  "120",
+					  1, 120));
+    op->hide();
+    handlers.push_back(op);
+  }
+  {
+    SharedHandle<OptionHandler> op(new BooleanOptionHandler
+				   (PREF_BT_HASH_CHECK_SEED,
+				    TEXT_BT_HASH_CHECK_SEED,
+				    V_TRUE,
+				    OptionHandler::OPT_ARG));
+    op->addTag(TAG_BITTORRENT);
     handlers.push_back(op);
   }
   {
@@ -707,6 +960,16 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_BITTORRENT);
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new NumberOptionHandler
+				   (PREF_BT_MAX_PEERS,
+				    TEXT_BT_MAX_PEERS,
+				    "55",
+				    0));
+    op->addTag(TAG_BITTORRENT);
+    handlers.push_back(op);
+				   
+  }
   {
     SharedHandle<OptionHandler> op(new ParameterOptionHandler
 				   (PREF_BT_MIN_CRYPTO_LEVEL,
@@ -716,6 +979,13 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_BITTORRENT);
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new PrioritizePieceOptionHandler
+				   (PREF_BT_PRIORITIZE_PIECE,
+				    TEXT_BT_PRIORITIZE_PIECE));
+    op->addTag(TAG_BITTORRENT);
+    handlers.push_back(op);
+  }
   {
     SharedHandle<OptionHandler> op(new UnitNumberOptionHandler
 				   (PREF_BT_REQUEST_PEER_SPEED_LIMIT,
@@ -734,36 +1004,47 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     handlers.push_back(op);
   }
   {
-    SharedHandle<OptionHandler> op(new NumberOptionHandler
-				   (PREF_BT_REQUEST_TIMEOUT,
-				    NO_DESCRIPTION,
-				    "60",
-				    1, 600,
-				    true));
+    SharedHandle<NumberOptionHandler> op(new NumberOptionHandler
+					 (PREF_BT_REQUEST_TIMEOUT,
+					  NO_DESCRIPTION,
+					  "60",
+					  1, 600));
+    op->hide();
     handlers.push_back(op);
   }
   {
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_BT_SEED_UNVERIFIED,
 				    TEXT_BT_SEED_UNVERIFIED,
-				    V_FALSE));
+				    V_FALSE,
+				    OptionHandler::OPT_ARG));
     op->addTag(TAG_BITTORRENT);
     handlers.push_back(op);
   }
   {
     SharedHandle<OptionHandler> op(new NumberOptionHandler
-				   (PREF_BT_TIMEOUT,
-				    NO_DESCRIPTION,
-				    "180",
-				    1, 600,
-				    true));
+				   (PREF_BT_STOP_TIMEOUT,
+				    TEXT_BT_STOP_TIMEOUT,
+				    "0",
+				    0));
+    op->addTag(TAG_BITTORRENT);
     handlers.push_back(op);
   }
   {
-    SharedHandle<OptionHandler> op(new BooleanOptionHandler
-				   (PREF_DIRECT_FILE_MAPPING,
-				    TEXT_DIRECT_FILE_MAPPING,
-				    V_TRUE));
+    SharedHandle<NumberOptionHandler> op(new NumberOptionHandler
+					 (PREF_BT_TIMEOUT,
+					  NO_DESCRIPTION,
+					  "180",
+					  1, 600));
+    op->hide();
+    handlers.push_back(op);
+  }
+  {
+    SharedHandle<NumberOptionHandler> op(new NumberOptionHandler
+					 (PREF_BT_TRACKER_INTERVAL,
+					  TEXT_BT_TRACKER_INTERVAL,
+					  "0",
+					  0));
     op->addTag(TAG_BITTORRENT);
     handlers.push_back(op);
   }
@@ -781,7 +1062,7 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new DefaultOptionHandler
 				   (PREF_DHT_FILE_PATH,
 				    TEXT_DHT_FILE_PATH,
-				    Util::getHomeDir()+"/.aria2/dht.dat",
+				    util::getHomeDir()+"/.aria2/dht.dat",
 				    "/PATH/TO/DHT_DAT"));
     op->addTag(TAG_BITTORRENT);
     handlers.push_back(op);
@@ -800,7 +1081,8 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_ENABLE_DHT,
 				    TEXT_ENABLE_DHT,
-				    V_FALSE));
+				    V_TRUE,
+				    OptionHandler::OPT_ARG));
     op->addTag(TAG_BASIC);
     op->addTag(TAG_BITTORRENT);
     handlers.push_back(op);
@@ -809,7 +1091,8 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_ENABLE_PEER_EXCHANGE,
 				    TEXT_ENABLE_PEER_EXCHANGE,
-				    V_TRUE));
+				    V_TRUE,
+				    OptionHandler::OPT_ARG));
     op->addTag(TAG_BITTORRENT);
     handlers.push_back(op);
   }
@@ -822,6 +1105,14 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_BITTORRENT);
     handlers.push_back(op);
   }
+  {
+    SharedHandle<OptionHandler> op(new IndexOutOptionHandler
+				   (PREF_INDEX_OUT,
+				    TEXT_INDEX_OUT,
+				    'O'));
+    op->addTag(TAG_BITTORRENT);
+    handlers.push_back(op);
+  }
   {
     SharedHandle<OptionHandler> op(new IntegerRangeOptionHandler
 				   (PREF_LISTEN_PORT,
@@ -847,25 +1138,26 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
 				   (PREF_MAX_UPLOAD_LIMIT,
 				    TEXT_MAX_UPLOAD_LIMIT,
 				    "0",
-				    0));
+				    0, -1,
+				    'u'));
     op->addTag(TAG_BASIC);
     op->addTag(TAG_BITTORRENT);
     handlers.push_back(op);
   }
   {
-    SharedHandle<OptionHandler> op(new NumberOptionHandler
-				   (PREF_PEER_CONNECTION_TIMEOUT,
-				    NO_DESCRIPTION,
-				    "20",
-				    1, 600,
-				    true));
+    SharedHandle<NumberOptionHandler> op(new NumberOptionHandler
+					 (PREF_PEER_CONNECTION_TIMEOUT,
+					  NO_DESCRIPTION,
+					  "20",
+					  1, 600));
+    op->hide();
     handlers.push_back(op);
   }
   {
     SharedHandle<OptionHandler> op(new DefaultOptionHandler
 				   (PREF_PEER_ID_PREFIX,
 				    TEXT_PEER_ID_PREFIX,
-				    "-aria2-"));
+				    "aria2/"PACKAGE_VERSION"-"));
     op->addTag(TAG_BITTORRENT);
     handlers.push_back(op);
   }
@@ -888,14 +1180,19 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     handlers.push_back(op);
   }
   {
-    SharedHandle<OptionHandler> op(new DefaultOptionHandler
+    SharedHandle<OptionHandler> op(new LocalFilePathOptionHandler
 				   (PREF_TORRENT_FILE,
-				    TEXT_TORRENT_FILE));
+				    TEXT_TORRENT_FILE,
+				    NO_DEFAULT_VALUE,
+				    false,
+				    'T'));
     op->addTag(TAG_BASIC);
     op->addTag(TAG_BITTORRENT);
     handlers.push_back(op);
   }
+#endif // ENABLE_BITTORRENT
   // Metalink Specific Options
+#ifdef ENABLE_METALINK
   {
     SharedHandle<OptionHandler> op(new ParameterOptionHandler
 				   (PREF_FOLLOW_METALINK,
@@ -909,14 +1206,18 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     SharedHandle<OptionHandler> op(new BooleanOptionHandler
 				   (PREF_METALINK_ENABLE_UNIQUE_PROTOCOL,
 				    TEXT_METALINK_ENABLE_UNIQUE_PROTOCOL,
-				    V_TRUE));
+				    V_TRUE,
+				    OptionHandler::OPT_ARG));
     op->addTag(TAG_METALINK);
     handlers.push_back(op);
   }
   {
-    SharedHandle<OptionHandler> op(new DefaultOptionHandler
+    SharedHandle<OptionHandler> op(new LocalFilePathOptionHandler
 				   (PREF_METALINK_FILE,
-				    TEXT_METALINK_FILE));
+				    TEXT_METALINK_FILE,
+				    NO_DEFAULT_VALUE,
+				    true,
+				    'M'));
     op->addTag(TAG_BASIC);
     op->addTag(TAG_METALINK);
     handlers.push_back(op);
@@ -958,7 +1259,8 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
 				   (PREF_METALINK_SERVERS,
 				    TEXT_METALINK_SERVERS,
 				    "5",
-				    1));
+				    1, -1,
+				    'C'));
     op->addTag(TAG_METALINK);
     handlers.push_back(op);
   }
@@ -969,21 +1271,44 @@ OptionHandlers OptionHandlerFactory::createOptionHandlers()
     op->addTag(TAG_METALINK);
     handlers.push_back(op);
   }
+#endif // ENABLE_METALINK
+  // Version Option
+  {
+    SharedHandle<OptionHandler> op(new DefaultOptionHandler
+				   ("version",
+				    TEXT_VERSION,
+				    NO_DEFAULT_VALUE,
+				    A2STR::NIL,
+				    OptionHandler::NO_ARG,
+				    'v'));
+    op->addTag(TAG_BASIC);
+    handlers.push_back(op);
+  }
   // Help Option
   {
     SharedHandle<OptionHandler> op(new DefaultOptionHandler
 				   ("help",
 				    TEXT_HELP,
 				    TAG_BASIC,
-				    StringFormat("%s,%s,%s,%s,%s,%s,%s,%s,all",
-						 TAG_BASIC,
-						 TAG_ADVANCED,
-						 TAG_HTTP,
-						 TAG_HTTPS,
-						 TAG_FTP,
-						 TAG_METALINK,
-						 TAG_BITTORRENT,
-						 TAG_HELP).str()));
+				    StringFormat
+				    ("%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, "
+				     "%s, %s, %s, %s",
+				     TAG_BASIC,
+				     TAG_ADVANCED,
+				     TAG_HTTP,
+				     TAG_HTTPS,
+				     TAG_FTP,
+				     TAG_METALINK,
+				     TAG_BITTORRENT,
+				     TAG_COOKIE,
+				     TAG_HOOK,
+				     TAG_FILE,
+				     TAG_XML_RPC,
+				     TAG_EXPERIMENTAL,
+				     TAG_HELP,
+				     TAG_ALL).str(),
+				    OptionHandler::OPT_ARG,
+				    'h'));
     op->addTag(TAG_BASIC);
     op->addTag(TAG_HELP);
     handlers.push_back(op);
