@@ -99,10 +99,15 @@ public:
 
 #endif // ENABLE_BITTORRENT
 
+  // Returns true if there is at least one missing and unused piece.
+  virtual bool hasMissingUnusedPiece() = 0;
+
   /**
    * Returns a missing piece if available. Otherwise returns 0;
+   * If ignoreBitfield is set, indexes of true bit are excluded.
    */
-  virtual SharedHandle<Piece> getMissingPiece() = 0;
+  virtual SharedHandle<Piece> getSparseMissingUnusedPiece
+  (const unsigned char* ignoreBitfield, size_t length) = 0;
 
   /**
    * Returns a missing piece whose index is index.
@@ -183,6 +188,8 @@ public:
 
   virtual bool isEndGame() = 0;
 
+  virtual void setEndGamePieceNum(size_t num) = 0;
+
   virtual SharedHandle<DiskAdaptor> getDiskAdaptor() = 0;
   
   virtual size_t getPieceLength(size_t index) = 0;
