@@ -36,8 +36,15 @@
 #define _D_SOCKET_CORE_H_
 
 #include "common.h"
+
+#ifdef HAVE_EPOLL
+# include <sys/epoll.h>
+#endif // HAVE_EPOLL
+
 #include <string>
+#include <cstdlib>
 #include <utility>
+#include <vector>
 
 #ifdef HAVE_LIBSSL
 // for SSL
@@ -48,68 +55,124 @@
 # include <gnutls/gnutls.h>
 #endif // HAVE_LIBGNUTLS
 
-using namespace std;
+#include "SharedHandle.h"
+#include "a2io.h"
+#include "a2netcompat.h"
+#include "a2time.h"
+
+namespace aria2 {
+
+#ifdef ENABLE_SSL
+class TLSContext;
+#endif // ENABLE_SSL
 
 class SocketCore {
   friend bool operator==(const SocketCore& s1, const SocketCore& s2);
   friend bool operator!=(const SocketCore& s1, const SocketCore& s2);
   friend bool operator<(const SocketCore& s1, const SocketCore& s2);
 private:
+  // socket type defined in <sys/socket.h>
+  int _sockType;
   // socket endpoint descriptor
-  int32_t sockfd;
-  // reference counter for this object.
-  int32_t use;
+  sock_t sockfd;
+
+#ifdef HAVE_EPOLL
+
+  // file descriptor used for epoll
+  int _epfd;
+
+  struct epoll_event _epEvent;
+
+#endif // HAVE_EPOLL
+
+  enum PollMethod {
+    POLL_METHOD_EPOLL, POLL_METHOD_SELECT
+  };
+
+  static PollMethod _pollMethod;
+
+  static int _protocolFamily;
+
+  static std::vector<std::pair<struct sockaddr_storage, socklen_t> > _bindAddrs;
+
   bool blocking;
-  bool secure;
+  int secure;
+
+  bool _wantRead;
+  bool _wantWrite;
+
+#if ENABLE_SSL
+  static SharedHandle<TLSContext> _tlsContext;
+#endif // ENABLE_SSL
+
 #ifdef HAVE_LIBSSL
   // for SSL
-  SSL_CTX* sslCtx;
   SSL* ssl;
+
+  int sslHandleEAGAIN(int ret);
 #endif // HAVE_LIBSSL
 #ifdef HAVE_LIBGNUTLS
   gnutls_session_t sslSession;
-  gnutls_certificate_credentials_t sslXcred;
   char* peekBuf;
-  int32_t peekBufLength;
-  int32_t peekBufMax;
+  size_t peekBufLength;
+  size_t peekBufMax;
 
-  int32_t shiftPeekData(char* data, int32_t len);
-  void addPeekData(char* data, int32_t len);
-  int32_t gnutlsRecv(char* data, int32_t len);
-  int32_t gnutlsPeek(char* data, int32_t len);
+  size_t shiftPeekData(char* data, size_t len);
+  void addPeekData(char* data, size_t len);
+  ssize_t gnutlsRecv(char* data, size_t len);
+  ssize_t gnutlsPeek(char* data, size_t len);
+
+  void gnutlsRecordCheckDirection();
 #endif // HAVE_LIBGNUTLS
 
   void init();
-  SocketCore(int32_t sockfd);
+
+  void bind(const struct sockaddr* addr, socklen_t addrlen);
+
+#ifdef HAVE_EPOLL
+
+  void initEPOLL();
+
+#endif // HAVE_EPOLL
+
+  SocketCore(sock_t sockfd, int sockType);
+
   static int error();
   static const char *errorMsg();
   static const char *errorMsg(const int err);
 public:
-  SocketCore();
+  SocketCore(int sockType = SOCK_STREAM);
   ~SocketCore();
 
-  int32_t getSockfd() const { return sockfd; }
+  sock_t getSockfd() const { return sockfd; }
 
-  bool isOpen() const { return sockfd != -1; }
+  bool isOpen() const { return sockfd != (sock_t) -1; }
 
   /**
-   * Creates a socket and listens form connection on it.
+   * Creates a socket and bind it with locahost's address and port.
+   * flags is set to struct addrinfo's ai_flags.
    * @param port port to listen. If 0 is specified, os automaticaly
    * choose avaiable port.
    */
-  void beginListen(int32_t port = 0);
+  void bind(uint16_t port, int flags = AI_PASSIVE);
+
+  /**
+   * Listens form connection on it.
+   * Call bind(uint16_t) before calling this function.
+   */
+  void beginListen();
 
   /**
    * Stores host address and port of this socket to addrinfo.
    * @param addrinfo placeholder to store host address and port.
    */
-  void getAddrInfo(pair<string, int32_t>& addrinfo) const;
-  
+  void getAddrInfo(std::pair<std::string, uint16_t>& addrinfo) const;
+
   /**
    * Stores peer's address and port to peerinfo.
    * @param peerinfo placeholder to store peer's address and port.
    */
-  void getPeerInfo(pair<string, int32_t>& peerinfo) const;
+  void getPeerInfo(std::pair<std::string, uint16_t>& peerinfo) const;
 
   /**
    * Accepts incoming connection on this socket.
@@ -126,7 +189,7 @@ public:
    * @param host hostname or ip address to connect to
    * @param port service port number to connect to
    */
-  void establishConnection(const string& host, int32_t port);
+  void establishConnection(const std::string& host, uint16_t port);
 
   void setNonBlockingMode();
 
@@ -147,7 +210,7 @@ public:
    * @return true if the socket is available for writing,
    * otherwise returns false.
    */
-  bool isWritable(int32_t timeout) const;
+  bool isWritable(time_t timeout);
 
   /**
    * Checks whether this socket is available for reading.
@@ -156,20 +219,38 @@ public:
    * @return true if the socket is available for reading,
    * otherwise returns false.
    */
-  bool isReadable(int32_t timeout) const;
+  bool isReadable(time_t timeout);
 
   /**
-   * Writes characters into this socket. data is a pointer pointing the first
+   * Writes data into this socket. data is a pointer pointing the first
    * byte of the data and len is the length of data.
-   * This method internally calls isWritable(). The parmeter timeout is used
-   * for this method call.
+   * If the underlying socket is in blocking mode, this method may block until
+   * all data is sent.
+   * If the underlying socket is in non-blocking mode, this method may return
+   * even if all data is sent. The size of written data is returned. If
+   * underlying socket gets EAGAIN, _wantRead or _wantWrite is set accordingly.
+   * This method sets _wantRead and _wantWrite to false before do anything else.
    * @param data data to write
    * @param len length of data
    */
-  void writeData(const char* data, int32_t len);
-  void writeData(const string& msg)
+  ssize_t writeData(const char* data, size_t len);
+  ssize_t writeData(const std::string& msg)
   {
-    writeData(msg.c_str(), msg.size());
+    return writeData(msg.c_str(), msg.size());
+  }
+  ssize_t writeData(const unsigned char* data, size_t len)
+  {
+    return writeData(reinterpret_cast<const char*>(data), len);
+  }
+
+  ssize_t writeData(const char* data, size_t len,
+		    const std::string& host, uint16_t port);
+
+  ssize_t writeData(const unsigned char* data, size_t len,
+		    const std::string& host,
+		    uint16_t port)
+  {
+    return writeData(reinterpret_cast<const char*>(data), len, host, port);
   }
 
   /**
@@ -178,13 +259,33 @@ public:
    * byte of the data, which must be allocated before this method is called.
    * len is the size of the allocated memory. When this method returns
    * successfully, len is replaced by the size of the read data.
-   * This method internally calls isReadable(). The parameter timeout is used
-   * for this method call.
+   * If the underlying socket is in blocking mode, this method may block until
+   * at least 1byte is received.
+   * If the underlying socket is in non-blocking mode, this method may return
+   * even if no single byte is received. If the underlying socket gets EAGAIN,
+   * _wantRead or _wantWrite is set accordingly.
+   * This method sets _wantRead and _wantWrite to false before do anything else.
    * @param data holder to store data.
    * @param len the maximum size data can store. This method assigns
    * the number of bytes read to len.
    */
-  void readData(char* data, int32_t& len);
+  void readData(char* data, size_t& len);
+
+  void readData(unsigned char* data, size_t& len)
+  {
+    readData(reinterpret_cast<char*>(data), len);
+  }
+
+  ssize_t readDataFrom(char* data, size_t len,
+		       std::pair<std::string /* numerichost */,
+		       uint16_t /* port */>& sender);
+
+  ssize_t readDataFrom(unsigned char* data, size_t len,
+		       std::pair<std::string /* numerichost */,
+		       uint16_t /* port */>& sender)
+  {
+    return readDataFrom(reinterpret_cast<char*>(data), len, sender);
+  }
 
   /**
    * Reads up to len bytes from this socket, but bytes are not removed from
@@ -195,25 +296,73 @@ public:
    * @param len the maximum size data can store. This method assigns
    * the number of bytes read to len.
    */
-  void peekData(char* data, int32_t& len);
-  
+  void peekData(char* data, size_t& len);
+
+  void peekData(unsigned char* data, size_t& len)
+  {
+    peekData(reinterpret_cast<char*>(data), len);
+  }
+
   /**
    * Makes this socket secure.
    * If the system has not OpenSSL, then this method do nothing.
    * connection must be established  before calling this method.
+   *
+   * If you are going to verify peer's certificate, hostname must be supplied.
    */
-  void initiateSecureConnection();
+  bool initiateSecureConnection(const std::string& hostname="");
+
+  void prepareSecureConnection();
 
   bool operator==(const SocketCore& s) {
     return sockfd == s.sockfd;
   }
-  
+
   bool operator!=(const SocketCore& s) {
     return !(*this == s);
   }
-  
+
   bool operator<(const SocketCore& s) {
     return sockfd < s.sockfd;
   }
+
+  std::string getSocketError() const;
+
+  /**
+   * Returns true if the underlying socket gets EAGAIN in the previous
+   * readData() or writeData() and the socket needs more incoming data to
+   * continue the operation.
+   */
+  bool wantRead() const;
+
+  /**
+   * Returns true if the underlying socket gets EAGAIN in the previous
+   * readData() or writeData() and the socket needs to write more data.
+   */
+  bool wantWrite() const;
+
+#ifdef HAVE_EPOLL
+  static void useEpoll();
+#endif // HAVE_EPOLL
+  static void useSelect();
+
+#ifdef ENABLE_SSL
+  static void setTLSContext(const SharedHandle<TLSContext>& tlsContext);
+#endif // ENABLE_SSL
+
+  static void setProtocolFamily(int protocolFamily)
+  {
+    _protocolFamily = protocolFamily;
+  }
+
+  // Bind socket to interface. interface may be specified as a
+  // hostname, IP address or interface name like eth0.  If the given
+  // interface is not found or binding socket is failed, exception
+  // will be thrown.  Set _protocolFamily before calling this function
+  // if you limit protocol family.
+  static void bindAddress(const std::string& interface);
 };
+
+} // namespace aria2
+
 #endif // _D_SOCKET_CORE_H_
