@@ -34,8 +34,9 @@
 /* copyright --> */
 #include "SpeedCalc.h"
 #include <algorithm>
-#include <ostream>
-#include <iterator>
+#include <functional>
+
+namespace aria2 {
 
 #define CHANGE_INTERVAL_SEC 15
 
@@ -47,8 +48,8 @@ public:
 };
 
 void SpeedCalc::reset() {
-  fill(&lengthArray[0], &lengthArray[2], 0);
-  for_each(&cpArray[0], &cpArray[2], Reset());
+  std::fill(&lengthArray[0], &lengthArray[2], 0);
+  std::for_each(&cpArray[0], &cpArray[2], Reset());
   sw = 0;
   maxSpeed = 0;
   prevSpeed = 0;
@@ -57,44 +58,40 @@ void SpeedCalc::reset() {
   nextInterval = CHANGE_INTERVAL_SEC;
 }
 
-int32_t SpeedCalc::calculateSpeed() {
-  int32_t milliElapsed = cpArray[sw].differenceInMillis();
+unsigned int SpeedCalc::calculateSpeed() {
+  int64_t milliElapsed = cpArray[sw].differenceInMillis();
   if(milliElapsed) {
-    int32_t speed = lengthArray[sw]*1000/milliElapsed;
+    unsigned int speed = lengthArray[sw]*1000/milliElapsed;
     prevSpeed = speed;
-    maxSpeed = max<int32_t>(speed, maxSpeed);
+    maxSpeed = std::max(speed, maxSpeed);
+    if(isIntervalOver(milliElapsed)) {
+      changeSw();
+    }
     return speed;
   } else {
     return prevSpeed;
   }
 }
 
-int32_t SpeedCalc::calculateSpeed(const struct timeval& now) {
+unsigned int SpeedCalc::calculateSpeed(const struct timeval& now) {
   int64_t milliElapsed = cpArray[sw].differenceInMillis(now);
   if(milliElapsed) {
-    int32_t speed = lengthArray[sw]*1000/milliElapsed;
+    unsigned int speed = lengthArray[sw]*1000/milliElapsed;
     prevSpeed = speed;
-    maxSpeed = max<int32_t>(speed, maxSpeed);
+    maxSpeed = std::max(speed, maxSpeed);
+    if(isIntervalOver(milliElapsed)) {
+      changeSw();
+    }
     return speed;
   } else {
     return prevSpeed;
   }
 }
 
-class Plus {
-private:
-  int32_t d;
-public:
-  Plus(int32_t d):d(d) {}
-
-  void operator()(int64_t& length) {
-    length += d;
-  }
-};
-
-void SpeedCalc::update(int bytes) {
+void SpeedCalc::update(size_t bytes) {
   accumulatedLength += bytes;
-  for_each(&lengthArray[0], &lengthArray[2], Plus(bytes));
+  std::transform(&lengthArray[0], &lengthArray[2], &lengthArray[0],
+		 std::bind1st(std::plus<uint64_t>(), (uint64_t)bytes));
   if(isIntervalOver()) {
     changeSw();
   }
@@ -104,6 +101,11 @@ bool SpeedCalc::isIntervalOver() const {
   return nextInterval <= cpArray[sw].difference();
 }
 
+bool SpeedCalc::isIntervalOver(int64_t milliElapsed) const
+{
+  return nextInterval <= milliElapsed/1000;
+}
+
 void SpeedCalc::changeSw() {
   lengthArray[sw] = 0;
   cpArray[sw].reset();
@@ -111,12 +113,16 @@ void SpeedCalc::changeSw() {
   nextInterval = cpArray[sw].difference()+CHANGE_INTERVAL_SEC;
 }
 
-int32_t SpeedCalc::getAvgSpeed() const {
-  int32_t milliElapsed = start.differenceInMillis();
-  if(milliElapsed) {
-    int32_t speed = accumulatedLength*1000/milliElapsed;
+unsigned int SpeedCalc::calculateAvgSpeed() const {
+  uint64_t milliElapsed = start.differenceInMillis();
+
+  // if milliElapsed is too small, the average speed is rubish, better return 0
+  if(milliElapsed > 4) {
+    unsigned int speed = accumulatedLength*1000/milliElapsed;
     return speed;
   } else {
     return 0;
   }
 }
+
+} // namespace aria2
