@@ -35,49 +35,63 @@
 #include "XML2SAXMetalinkProcessor.h"
 #include "BinaryStream.h"
 #include "MetalinkParserStateMachine.h"
-#include "Util.h"
+#include "Metalinker.h"
+#include "MetalinkEntry.h"
+#include "util.h"
 #include "message.h"
+#include "DlAbortEx.h"
+
+namespace aria2 {
 
 class SessionData {
 public:
-  MetalinkParserStateMachineHandle _stm;
+  SharedHandle<MetalinkParserStateMachine> _stm;
 
-  Strings _charactersStack;
+  std::deque<std::string> _charactersStack;
 
-  SessionData(const MetalinkParserStateMachineHandle& stm):_stm(stm) {}
+  SessionData(const SharedHandle<MetalinkParserStateMachine>& stm):_stm(stm) {}
 };
 
-typedef SharedHandle<SessionData> SessionDataHandle;
-
 static void mlStartElement(void* userData, const xmlChar* name, const xmlChar** attrs)
 {
-  ((SessionData*)userData)->_charactersStack.push_front(string());
-  map<string, string> attrmap;
+  SessionData* sd = reinterpret_cast<SessionData*>(userData);
+  std::map<std::string, std::string> attrmap;
   if(attrs) {
     const xmlChar** p = attrs;
     while(*p != 0) {
-      string name = (const char*)*p++;
+      std::string name = reinterpret_cast<const char*>(*p);
+      ++p;
       if(*p == 0) {
 	break;
       }
-      string value = Util::trim((const char*)*p++);
+      std::string value = util::trim(reinterpret_cast<const char*>(*p));
+      ++p;
       attrmap[name] = value;
     }
   }
-  ((SessionData*)userData)->_stm->beginElement((const char*)name, attrmap);
+  sd->_stm->beginElement(reinterpret_cast<const char*>(name), attrmap);
+  if(sd->_stm->needsCharactersBuffering()) {
+    sd->_charactersStack.push_front(std::string());
+  }
 }
 
 static void mlEndElement(void* userData, const xmlChar* name)
 {
-  SessionData* sd = (SessionData*)userData;
-
-  sd->_stm->endElement((const char*)name, Util::trim(sd->_charactersStack.front()));
-  sd->_charactersStack.pop_front();
+  SessionData* sd = reinterpret_cast<SessionData*>(userData);
+  std::string characters;
+  if(sd->_stm->needsCharactersBuffering()) {
+    characters = util::trim(sd->_charactersStack.front());
+    sd->_charactersStack.pop_front();
+  }
+  sd->_stm->endElement(reinterpret_cast<const char*>(name), characters);
 }
 
 static void mlCharacters(void* userData, const xmlChar* ch, int len)
 {
-  ((SessionData*)userData)->_charactersStack.front() += string(&ch[0], &ch[len]);
+  SessionData* sd = reinterpret_cast<SessionData*>(userData);
+  if(sd->_stm->needsCharactersBuffering()) {
+    sd->_charactersStack.front() += std::string(&ch[0], &ch[len]);
+  }
 }
 
 static xmlSAXHandler mySAXHandler =
@@ -116,57 +130,58 @@ static xmlSAXHandler mySAXHandler =
   0, //   xmlStructuredErrorFunc
 };
 
-XML2SAXMetalinkProcessor::XML2SAXMetalinkProcessor():
-  _stm(0)
-{}
-	 
-	 
-MetalinkerHandle XML2SAXMetalinkProcessor::parseFile(const string& filename)
+SharedHandle<Metalinker>
+MetalinkProcessor::parseFile(const std::string& filename)
 {
-  _stm = new MetalinkParserStateMachine();
-  SessionDataHandle sessionData = new SessionData(_stm);
-  int32_t retval = xmlSAXUserParseFile(&mySAXHandler, sessionData.get(),
-				       filename.c_str());
+  _stm.reset(new MetalinkParserStateMachine());
+  SharedHandle<SessionData> sessionData(new SessionData(_stm));
+  int retval = xmlSAXUserParseFile(&mySAXHandler, sessionData.get(),
+				   filename.c_str());
   if(retval != 0) {
-    throw new DlAbortEx(MSG_CANNOT_PARSE_METALINK);
+    throw DL_ABORT_EX(MSG_CANNOT_PARSE_METALINK);
   }
   return _stm->getResult();
 }
 	 
-MetalinkerHandle XML2SAXMetalinkProcessor::parseFromBinaryStream(const BinaryStreamHandle& binaryStream)
+SharedHandle<Metalinker>
+MetalinkProcessor::parseFromBinaryStream(const SharedHandle<BinaryStream>& binaryStream)
 {
-  _stm = new MetalinkParserStateMachine();
-  int32_t bufSize = 4096;
+  _stm.reset(new MetalinkParserStateMachine());
+  size_t bufSize = 4096;
   unsigned char buf[bufSize];
 
-  int32_t res = binaryStream->readData(buf, 4, 0);
+  ssize_t res = binaryStream->readData(buf, 4, 0);
   if(res != 4) {
-    throw new DlAbortEx("Too small data for parsing XML.");
+    throw DL_ABORT_EX("Too small data for parsing XML.");
   }
 
-  SessionDataHandle sessionData = new SessionData(_stm);
-  xmlParserCtxtPtr ctx = xmlCreatePushParserCtxt(&mySAXHandler, sessionData.get(), (const char*)buf, res, 0);
+  SharedHandle<SessionData> sessionData(new SessionData(_stm));
+  xmlParserCtxtPtr ctx = xmlCreatePushParserCtxt
+    (&mySAXHandler, sessionData.get(),
+     reinterpret_cast<const char*>(buf), res, 0);
   try {
-    int64_t readOffset = res;
+    off_t readOffset = res;
     while(1) {
-      int32_t res = binaryStream->readData(buf, bufSize, readOffset);
+      ssize_t res = binaryStream->readData(buf, bufSize, readOffset);
       if(res == 0) {
 	break;
       }
-      if(xmlParseChunk(ctx, (const char*)buf, res, 0) != 0) {
-	throw new DlAbortEx(MSG_CANNOT_PARSE_METALINK);
+      if(xmlParseChunk(ctx, reinterpret_cast<const char*>(buf), res, 0) != 0) {
+	throw DL_ABORT_EX(MSG_CANNOT_PARSE_METALINK);
       }
       readOffset += res;
     }
-    xmlParseChunk(ctx, (const char*)buf, 0, 1);
-  } catch(Exception* e) {
+    xmlParseChunk(ctx, reinterpret_cast<const char*>(buf), 0, 1);
+  } catch(Exception& e) {
     xmlFreeParserCtxt(ctx);
-    throw e;
+    throw;
   }
   xmlFreeParserCtxt(ctx);
 
   if(!_stm->finished()) {
-    throw new DlAbortEx(MSG_CANNOT_PARSE_METALINK);
+    throw DL_ABORT_EX(MSG_CANNOT_PARSE_METALINK);
   }
   return _stm->getResult();
 }
+
+} // namespace aria2
