@@ -48,8 +48,6 @@ const std::string V_ALL("all");
  * General preferences
  */
 // values: 1*digit
-const std::string PREF_RETRY_WAIT("retry-wait");
-// values: 1*digit
 const std::string PREF_TIMEOUT("timeout");
 // values: 1*digit
 const std::string PREF_DNS_TIMEOUT("dns-timeout");
@@ -158,8 +156,6 @@ const std::string PREF_XML_RPC_LISTEN_PORT("xml-rpc-listen-port");
 // value: true | false
 const std::string PREF_ENABLE_XML_RPC("enable-xml-rpc");
 // value: true | false
-const std::string PREF_RESET_URI("reset-uri");
-// value: true | false
 const std::string PREF_DRY_RUN("dry-run");
 // value: true | false
 const std::string PREF_REUSE_URI("reuse-uri");
@@ -174,6 +170,12 @@ const std::string PREF_ON_DOWNLOAD_START("on-download-start");
 const std::string PREF_ON_DOWNLOAD_STOP("on-download-stop");
 const std::string PREF_ON_DOWNLOAD_COMPLETE("on-download-complete");
 const std::string PREF_ON_DOWNLOAD_ERROR("on-download-error");
+// value: true | false
+const std::string PREF_XML_RPC_LISTEN_ALL("xml-rpc-listen-all");
+// value: string
+const std::string PREF_INTERFACE("interface");
+// value: true | false
+const std::string PREF_DISABLE_IPV6("disable-ipv6");
 
 /**
  * FTP related preferences
@@ -194,9 +196,6 @@ const std::string PREF_FTP_REUSE_CONNECTION("ftp-reuse-connection");
  */
 const std::string PREF_HTTP_USER("http-user");
 const std::string PREF_HTTP_PASSWD("http-passwd");
-// values: basic
-const std::string PREF_HTTP_AUTH_SCHEME("http-auth-scheme");
-const std::string V_BASIC("basic");
 // values: string
 const std::string PREF_USER_AGENT("user-agent");
 // value: string that your file system recognizes as a file name.
@@ -305,6 +304,10 @@ const std::string PREF_BT_EXTERNAL_IP("bt-external-ip");
 const std::string PREF_INDEX_OUT("index-out");
 // values: 1*digit
 const std::string PREF_BT_TRACKER_INTERVAL("bt-tracker-interval");
+// values: 1*digit
+const std::string PREF_BT_STOP_TIMEOUT("bt-stop-timeout");
+// values: head[=SIZE]|tail[=SIZE], ...
+const std::string PREF_BT_PRIORITIZE_PIECE("bt-prioritize-piece");
 
 /**
  * Metalink related preferences
