@@ -33,6 +33,11 @@
  */
 /* copyright --> */
 #include "common.h"
+
+#include <iostream>
+#include <iterator>
+#include <algorithm>
+
 #include "SharedHandle.h"
 #include "a2io.h"
 #include "FeatureConfig.h"
@@ -44,9 +49,7 @@
 #include "StringFormat.h"
 #include "OptionParser.h"
 #include "OptionHandler.h"
-#include <iostream>
-#include <iterator>
-#include <algorithm>
+#include "util.h"
 
 namespace aria2 {
 
@@ -55,10 +58,16 @@ void showVersion() {
 	    << "Copyright (C) 2006, 2009 Tatsuhiro Tsujikawa" << "\n"
 	    << "\n"
 	    <<
-      "This program is distributed in the hope that it will be useful,\n"
-      "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
-      "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
-      "GNU General Public License for more details.\n"
+    "This program is free software; you can redistribute it and/or modify\n"
+    "it under the terms of the GNU General Public License as published by\n"
+    "the Free Software Foundation; either version 2 of the License, or\n"
+    "(at your option) any later version.\n"
+	    << "\n"
+	    <<
+    "This program is distributed in the hope that it will be useful,\n"
+    "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+    "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+    "GNU General Public License for more details.\n"
 	    << "\n"
 	    << "** Configuration **" << "\n"
 	    << "Enabled Features: "
@@ -68,35 +77,18 @@ void showVersion() {
 	    << MessageDigestContext::getSupportedAlgoString() << "\n"
 #endif // ENABLE_MESSAGE_DIGEST
 	    << "\n"
-	    << StringFormat(_("Report bugs to %s"), "<tujikawa at users dot sourceforge dot net>")
-	    << std::endl;
+	    << StringFormat(_("Report bugs to %s"), PACKAGE_BUGREPORT) << "\n"
+	    << "Visit " << PACKAGE_URL << std::endl;
 }
 
 void showUsage(const std::string& keyword, const OptionParser& oparser) {
-  std::cout << StringFormat(_("Usage: %s [OPTIONS] [URL | TORRENT_FILE |"
-			      " METALINK_FILE]..."), PACKAGE_NAME) << "\n"
+  std::cout << _("Usage: aria2c [OPTIONS] [URI | MAGNET | TORRENT_FILE |"
+		 " METALINK_FILE]...") << "\n"
 	    << "\n";
-
-  std::deque<SharedHandle<OptionHandler> > handlers =
-    keyword == V_ALL ? oparser.findAll():oparser.findByTag(keyword);
-  if(handlers.empty()) {
+  if(util::startsWith(keyword, "#")) {
     std::deque<SharedHandle<OptionHandler> > handlers =
-      oparser.findByNameSubstring(keyword);
-    if(!handlers.empty()) {
-      std::cout << StringFormat(_("Printing options whose name includes"
-				  " '%s'."), keyword.c_str())
-		<< "\n"
-		<< _("Options:") << "\n";
-      std::copy(handlers.begin(), handlers.end(),
-		std::ostream_iterator<SharedHandle<OptionHandler> >
-		(std::cout, "\n\n"));
-    } else {
-      std::cout << StringFormat(_("No help category or option name matching"
-				  " with '%s'."), keyword.c_str())
-		<< "\n" << oparser.findByName("help") << "\n";
-    }
-  } else {
-    if(keyword == V_ALL) {
+      keyword == TAG_ALL ? oparser.findAll():oparser.findByTag(keyword);
+    if(keyword == TAG_ALL) {
       std::cout << _("Printing all options.");
     } else {
       std::cout << StringFormat(_("Printing options tagged with '%s'."),
@@ -113,23 +105,41 @@ void showUsage(const std::string& keyword, const OptionParser& oparser) {
     std::copy(handlers.begin(), handlers.end(),
 	      std::ostream_iterator<SharedHandle<OptionHandler> >
 	      (std::cout, "\n\n"));
+  } else {    
+    std::deque<SharedHandle<OptionHandler> > handlers =
+      oparser.findByNameSubstring(keyword);
+    if(!handlers.empty()) {
+      std::cout << StringFormat(_("Printing options whose name includes"
+				  " '%s'."), keyword.c_str())
+		<< "\n"
+		<< _("Options:") << "\n";
+      std::copy(handlers.begin(), handlers.end(),
+		std::ostream_iterator<SharedHandle<OptionHandler> >
+		(std::cout, "\n\n"));
+    } else {
+      std::cout << StringFormat(_("No option matching with '%s'."),
+				keyword.c_str())
+		<< "\n" << oparser.findByName("help") << "\n";
+    }
   }
 
   if(keyword == TAG_BASIC) {
-    std::cout << "URL, TORRENT_FILE, METALINK_FILE:" << "\n"
-	      << _(" You can specify multiple URLs. Unless you specify -Z option, all URLs must\n"
-		   " point to the same file or downloading will fail.") << "\n"
-	      << _(" You can also specify arbitrary number of torrent files and metalink files\n"
-		   " stored in a local drive. Please note that they are always treated as a\n"
-		   " separate download.") << "\n"
+    std::cout << "URI, MAGNET, TORRENT_FILE, METALINK_FILE:" << "\n"
+	      << _(" You can specify multiple HTTP(S)/FTP URIs. Unless you specify -Z option, all\n"
+		   " URIs must point to the same file or downloading will fail.") << "\n"
+	      << _(" You can also specify arbitrary number of BitTorrent Magnet URIs, torrent/\n"
+		   " metalink files stored in a local drive. Please note that they are always\n"
+		   " treated as a separate download.") << "\n"
 
 	      << "\n"
-	      << _(" You can specify both torrent file with -T option and URLs. By doing this,\n"
-		   " download a file from both torrent swarm and http/ftp server at the same time,\n"
-		   " while the data from http/ftp are uploaded to the torrent swarm. Note that\n"
-		   " only single file torrent can be integrated with http/ftp.") << "\n"
+	      << _(" You can specify both torrent file with -T option and URIs. By doing this,\n"
+		   " download a file from both torrent swarm and HTTP/FTP server at the same time,\n"
+		   " while the data from HTTP/FTP are uploaded to the torrent swarm. For single file\n"
+		   " torrents, URI can be a complete URI pointing to the resource or if URI ends\n"
+		   " with '/', 'name' in torrent file is added. For multi-file torrents, 'name' and\n"
+		   " 'path' in torrent are added to form a URI for each file.") << "\n"
 	      << "\n"
-	      << _(" Make sure that URL is quoted with single(\') or double(\") quotation if it\n"
+	      << _(" Make sure that URI is quoted with single(\') or double(\") quotation if it\n"
 		   " contains \"&\" or any characters that have special meaning in shell.") << "\n"
 	      << "\n";
   }
