@@ -1,7 +1,8 @@
 #include "BtKeepAliveMessage.h"
+#include <cstring>
 #include <cppunit/extensions/HelperMacros.h>
 
-using namespace std;
+namespace aria2 {
 
 class BtKeepAliveMessageTest:public CppUnit::TestFixture {
 
@@ -25,12 +26,14 @@ void BtKeepAliveMessageTest::testGetMessage() {
   char msg[4];
   memset(msg, 0, sizeof(msg));
   BtKeepAliveMessage message;
-  CPPUNIT_ASSERT_EQUAL((int8_t)99, message.getId());
-  CPPUNIT_ASSERT_EQUAL((int32_t)4, message.getMessageLength());
+  CPPUNIT_ASSERT_EQUAL((uint8_t)99, message.getId());
+  CPPUNIT_ASSERT_EQUAL((size_t)4, message.getMessageLength());
   CPPUNIT_ASSERT(memcmp(msg, message.getMessage(), 4) == 0);
 }
 
 void BtKeepAliveMessageTest::testToString() {
   BtKeepAliveMessage msg;
-  CPPUNIT_ASSERT_EQUAL(string("keep alive"), msg.toString());
+  CPPUNIT_ASSERT_EQUAL(std::string("keep alive"), msg.toString());
 }
+
+} // namespace aria2
