@@ -1,14 +1,22 @@
 #include "BtPieceMessage.h"
-#include "PeerMessageUtil.h"
-#include "MockBtContext.h"
+
+#include <cstring>
+
+#include <cppunit/extensions/HelperMacros.h>
+
+#include "bittorrent_helper.h"
 #include "MockBtMessage.h"
 #include "MockBtMessageFactory.h"
 #include "MockBtMessageDispatcher.h"
 #include "BtChokingEvent.h"
 #include "BtCancelSendingPieceEvent.h"
-#include <cppunit/extensions/HelperMacros.h>
+#include "FileEntry.h"
+#include "Peer.h"
+#include "Piece.h"
+#include "BtHandshakeMessage.h"
+#include "DownloadContext.h"
 
-using namespace std;
+namespace aria2 {
 
 class BtPieceMessageTest:public CppUnit::TestFixture {
 
@@ -26,10 +34,9 @@ class BtPieceMessageTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testCancelSendingPieceEvent_invalidate);
   CPPUNIT_TEST(testCancelSendingPieceEvent_sendingInProgress);
   CPPUNIT_TEST(testToString);
+
   CPPUNIT_TEST_SUITE_END();
 public:
-  BtPieceMessageTest():btMessageDispatcher(0), peer(0), msg(0) {}
-
   void testCreate();
   void testGetMessageHeader();
   void testChokingEvent();
@@ -46,56 +53,49 @@ public:
 
   class MockBtMessage2 : public MockBtMessage {
   public:
-    int32_t index;
-    int32_t begin;
-    int32_t length;
+    size_t index;
+    uint32_t begin;
+    size_t length;
   public:
-    MockBtMessage2(int32_t index, int32_t begin, int32_t length):index(index), begin(begin), length(length) {}
+    MockBtMessage2(size_t index, uint32_t begin, size_t length):index(index), begin(begin), length(length) {}
 
   };
 
-  typedef SharedHandle<MockBtMessage2> MockBtMessage2Handle;
-
   class MockBtMessageFactory2 : public MockBtMessageFactory {
   public:
-    virtual BtMessageHandle createRejectMessage(int32_t index,
-						int32_t begin,
-						int32_t length) {
-      MockBtMessage2Handle msg = new MockBtMessage2(index, begin, length);
+    virtual SharedHandle<BtMessage>
+    createRejectMessage(size_t index,
+			uint32_t begin,
+			size_t length) {
+      SharedHandle<MockBtMessage2> msg(new MockBtMessage2(index, begin, length));
       return msg;
     }
   };
 
-  typedef SharedHandle<MockBtMessageFactory2> MockBtMessageFactory2Handle;
-
-  MockBtMessageDispatcherHandle btMessageDispatcher;
-  PeerHandle peer;
-  BtPieceMessageHandle msg;
+  SharedHandle<DownloadContext> _dctx;
+  SharedHandle<MockBtMessageDispatcher> btMessageDispatcher;
+  SharedHandle<MockBtMessageFactory> _btMessageFactory;
+  SharedHandle<Peer> peer;
+  SharedHandle<BtPieceMessage> msg;
 
   void setUp() {
-    BtRegistry::unregisterAll();
+    _dctx.reset(new DownloadContext(16*1024, 256*1024, "/path/to/file"));
 
-    MockBtContextHandle btContext;
-    btContext->setInfoHash((const unsigned char*)"12345678901234567890");
-    btContext->setPieceLength(16*1024);
-    btContext->setTotalLength(256*1024);
+    peer.reset(new Peer("host", 6969));
+    peer->allocateSessionResource(_dctx->getPieceLength(),
+				  _dctx->getTotalLength());
 
-    peer = new Peer("host", 6969, 16*1024, 256*1024);
-    BtRegistry::registerPeerObjectCluster(btContext->getInfoHashAsString(),
-					  new PeerObjectCluster());
-    PEER_OBJECT_CLUSTER(btContext)->registerHandle(peer->getId(), new PeerObject());
-    btMessageDispatcher = new MockBtMessageDispatcher();
-    PEER_OBJECT(btContext, peer)->btMessageDispatcher = btMessageDispatcher;
-    PEER_OBJECT(btContext, peer)->btMessageFactory = new MockBtMessageFactory2();
+    btMessageDispatcher.reset(new MockBtMessageDispatcher());
+    _btMessageFactory.reset(new MockBtMessageFactory2());
 
-    msg = new BtPieceMessage();
+    msg.reset(new BtPieceMessage());
     msg->setIndex(1);
     msg->setBegin(1024);
     msg->setBlockLength(16*1024);
-    msg->setBtContext(btContext);
+    msg->setDownloadContext(_dctx);
     msg->setPeer(peer);
     msg->setBtMessageDispatcher(btMessageDispatcher);
-    msg->setBtMessageFactory(BT_MESSAGE_FACTORY(btContext, peer));
+    msg->setBtMessageFactory(_btMessageFactory);
   }
 };
 
@@ -106,21 +106,21 @@ void BtPieceMessageTest::testCreate() {
   unsigned char msg[13+2];
   unsigned char data[2];
   memset(data, 0xff, sizeof(data));
-  PeerMessageUtil::createPeerMessageString(msg, sizeof(msg), 11, 7);
-  PeerMessageUtil::setIntParam(&msg[5], 12345);
-  PeerMessageUtil::setIntParam(&msg[9], 256);
+  bittorrent::createPeerMessageString(msg, sizeof(msg), 11, 7);
+  bittorrent::setIntParam(&msg[5], 12345);
+  bittorrent::setIntParam(&msg[9], 256);
   memcpy(&msg[13], data, sizeof(data));
-  BtPieceMessageHandle pm = BtPieceMessage::create(&msg[4], 11);
-  CPPUNIT_ASSERT_EQUAL((int8_t)7, pm->getId());
-  CPPUNIT_ASSERT_EQUAL((int32_t)12345, pm->getIndex());
-  CPPUNIT_ASSERT_EQUAL((int32_t)256, pm->getBegin());
+  SharedHandle<BtPieceMessage> pm = BtPieceMessage::create(&msg[4], 11);
+  CPPUNIT_ASSERT_EQUAL((uint8_t)7, pm->getId());
+  CPPUNIT_ASSERT_EQUAL((size_t)12345, pm->getIndex());
+  CPPUNIT_ASSERT_EQUAL((uint32_t)256, pm->getBegin());
   CPPUNIT_ASSERT(memcmp(data, pm->getBlock(), sizeof(data)) == 0);
-  CPPUNIT_ASSERT_EQUAL((int32_t)2, pm->getBlockLength());
+  CPPUNIT_ASSERT_EQUAL((size_t)2, pm->getBlockLength());
 
   // case: payload size is wrong
   try {
     unsigned char msg[13];
-    PeerMessageUtil::createPeerMessageString(msg, sizeof(msg), 9, 7);
+    bittorrent::createPeerMessageString(msg, sizeof(msg), 9, 7);
     BtPieceMessage::create(&msg[4], 9);
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(...) {
@@ -128,7 +128,7 @@ void BtPieceMessageTest::testCreate() {
   // case: id is wrong
   try {
     unsigned char msg[13+2];
-    PeerMessageUtil::createPeerMessageString(msg, sizeof(msg), 11, 8);
+    bittorrent::createPeerMessageString(msg, sizeof(msg), 11, 8);
     BtPieceMessage::create(&msg[4], 11);
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(...) {
@@ -141,9 +141,9 @@ void BtPieceMessageTest::testGetMessageHeader() {
   msg.setBegin(256);
   msg.setBlockLength(1024);
   unsigned char data[13];
-  PeerMessageUtil::createPeerMessageString(data, sizeof(data), 9+1024, 7);
-  PeerMessageUtil::setIntParam(&data[5], 12345);
-  PeerMessageUtil::setIntParam(&data[9], 256);
+  bittorrent::createPeerMessageString(data, sizeof(data), 9+1024, 7);
+  bittorrent::setIntParam(&data[5], 12345);
+  bittorrent::setIntParam(&data[9], 256);
   CPPUNIT_ASSERT(memcmp(msg.getMessageHeader(), data, 13) == 0);
 }
 
@@ -153,7 +153,7 @@ void BtPieceMessageTest::testChokingEvent() {
   CPPUNIT_ASSERT(!peer->isInAmAllowedIndexSet(1));
   CPPUNIT_ASSERT(!peer->isFastExtensionEnabled());
 
-  msg->handleEvent(new BtChokingEvent());
+  msg->onChokingEvent(BtChokingEvent());
 
   CPPUNIT_ASSERT(msg->isInvalidate());
   CPPUNIT_ASSERT_EQUAL((size_t)0, btMessageDispatcher->messageQueue.size());
@@ -167,14 +167,16 @@ void BtPieceMessageTest::testChokingEvent_allowedFastEnabled() {
   CPPUNIT_ASSERT(!peer->isInAmAllowedIndexSet(1));
   CPPUNIT_ASSERT(peer->isFastExtensionEnabled());
 
-  msg->handleEvent(new BtChokingEvent());
+  msg->onChokingEvent(BtChokingEvent());
 
   CPPUNIT_ASSERT(msg->isInvalidate());  
   CPPUNIT_ASSERT_EQUAL((size_t)1, btMessageDispatcher->messageQueue.size());
-  MockBtMessage2* rej = (MockBtMessage2*)btMessageDispatcher->messageQueue.front().get();
-  CPPUNIT_ASSERT_EQUAL((int32_t)1, rej->index);
-  CPPUNIT_ASSERT_EQUAL((int32_t)1024, rej->begin);
-  CPPUNIT_ASSERT_EQUAL((int32_t)16*1024, rej->length);
+  SharedHandle<MockBtMessage2> rej =
+    dynamic_pointer_cast<MockBtMessage2>
+    (btMessageDispatcher->messageQueue.front());
+  CPPUNIT_ASSERT_EQUAL((size_t)1, rej->index);
+  CPPUNIT_ASSERT_EQUAL((uint32_t)1024, rej->begin);
+  CPPUNIT_ASSERT_EQUAL((size_t)16*1024, rej->length);
 }
 
 void BtPieceMessageTest::testChokingEvent_inAmAllowedIndexSet() {
@@ -186,7 +188,7 @@ void BtPieceMessageTest::testChokingEvent_inAmAllowedIndexSet() {
   CPPUNIT_ASSERT(peer->isInAmAllowedIndexSet(1));
   CPPUNIT_ASSERT(peer->isFastExtensionEnabled());
 
-  msg->handleEvent(new BtChokingEvent());
+  msg->onChokingEvent(BtChokingEvent());
 
   CPPUNIT_ASSERT(!msg->isInvalidate());  
   CPPUNIT_ASSERT_EQUAL((size_t)0, btMessageDispatcher->messageQueue.size());
@@ -199,7 +201,7 @@ void BtPieceMessageTest::testChokingEvent_invalidate() {
   CPPUNIT_ASSERT(!peer->isInAmAllowedIndexSet(1));
   CPPUNIT_ASSERT(!peer->isFastExtensionEnabled());
 
-  msg->handleEvent(new BtChokingEvent());
+  msg->onChokingEvent(BtChokingEvent());
 
   CPPUNIT_ASSERT(msg->isInvalidate());  
   CPPUNIT_ASSERT_EQUAL((size_t)0, btMessageDispatcher->messageQueue.size());
@@ -212,7 +214,7 @@ void BtPieceMessageTest::testChokingEvent_sendingInProgress() {
   CPPUNIT_ASSERT(!peer->isInAmAllowedIndexSet(1));
   CPPUNIT_ASSERT(!peer->isFastExtensionEnabled());
 
-  msg->handleEvent(new BtChokingEvent());
+  msg->onChokingEvent(BtChokingEvent());
 
   CPPUNIT_ASSERT(!msg->isInvalidate());  
   CPPUNIT_ASSERT_EQUAL((size_t)0, btMessageDispatcher->messageQueue.size());
@@ -223,7 +225,7 @@ void BtPieceMessageTest::testCancelSendingPieceEvent() {
   CPPUNIT_ASSERT(!msg->isSendingInProgress());
   CPPUNIT_ASSERT(!peer->isFastExtensionEnabled());
 
-  msg->handleEvent(new BtCancelSendingPieceEvent(1, 1024, 16*1024));
+  msg->onCancelSendingPieceEvent(BtCancelSendingPieceEvent(1, 1024, 16*1024));
 
   CPPUNIT_ASSERT(msg->isInvalidate());
 }
@@ -233,15 +235,15 @@ void BtPieceMessageTest::testCancelSendingPieceEvent_noMatch() {
   CPPUNIT_ASSERT(!msg->isSendingInProgress());
   CPPUNIT_ASSERT(!peer->isFastExtensionEnabled());
 
-  msg->handleEvent(new BtCancelSendingPieceEvent(0, 1024, 16*1024));
+  msg->onCancelSendingPieceEvent(BtCancelSendingPieceEvent(0, 1024, 16*1024));
+
+  CPPUNIT_ASSERT(!msg->isInvalidate());
+  
+  msg->onCancelSendingPieceEvent(BtCancelSendingPieceEvent(1, 0, 16*1024));
 
   CPPUNIT_ASSERT(!msg->isInvalidate());
 
-  msg->handleEvent(new BtCancelSendingPieceEvent(1, 0, 16*1024));
-
-  CPPUNIT_ASSERT(!msg->isInvalidate());
-
-  msg->handleEvent(new BtCancelSendingPieceEvent(1, 1024, 0));
+  msg->onCancelSendingPieceEvent(BtCancelSendingPieceEvent(1, 1024, 0));
 
   CPPUNIT_ASSERT(!msg->isInvalidate());
 }
@@ -252,14 +254,16 @@ void BtPieceMessageTest::testCancelSendingPieceEvent_allowedFastEnabled() {
   CPPUNIT_ASSERT(!msg->isSendingInProgress());
   CPPUNIT_ASSERT(peer->isFastExtensionEnabled());
 
-  msg->handleEvent(new BtCancelSendingPieceEvent(1, 1024, 16*1024));
+  msg->onCancelSendingPieceEvent(BtCancelSendingPieceEvent(1, 1024, 16*1024));
 
   CPPUNIT_ASSERT(msg->isInvalidate());
   CPPUNIT_ASSERT_EQUAL((size_t)1, btMessageDispatcher->messageQueue.size());
-  MockBtMessage2* rej = (MockBtMessage2*)btMessageDispatcher->messageQueue.front().get();
-  CPPUNIT_ASSERT_EQUAL((int32_t)1, rej->index);
-  CPPUNIT_ASSERT_EQUAL((int32_t)1024, rej->begin);
-  CPPUNIT_ASSERT_EQUAL((int32_t)16*1024, rej->length);
+  SharedHandle<MockBtMessage2> rej =
+    dynamic_pointer_cast<MockBtMessage2>
+    (btMessageDispatcher->messageQueue.front());
+  CPPUNIT_ASSERT_EQUAL((size_t)1, rej->index);
+  CPPUNIT_ASSERT_EQUAL((uint32_t)1024, rej->begin);
+  CPPUNIT_ASSERT_EQUAL((size_t)16*1024, rej->length);
 }
 
 void BtPieceMessageTest::testCancelSendingPieceEvent_invalidate() {
@@ -269,7 +273,7 @@ void BtPieceMessageTest::testCancelSendingPieceEvent_invalidate() {
   CPPUNIT_ASSERT(!msg->isSendingInProgress());
   CPPUNIT_ASSERT(peer->isFastExtensionEnabled());
 
-  msg->handleEvent(new BtCancelSendingPieceEvent(1, 1024, 16*1024));
+  msg->onCancelSendingPieceEvent(BtCancelSendingPieceEvent(1, 1024, 16*1024));
 
   CPPUNIT_ASSERT(msg->isInvalidate());
   CPPUNIT_ASSERT_EQUAL((size_t)0, btMessageDispatcher->messageQueue.size());
@@ -281,12 +285,14 @@ void BtPieceMessageTest::testCancelSendingPieceEvent_sendingInProgress() {
   CPPUNIT_ASSERT(msg->isSendingInProgress());
   CPPUNIT_ASSERT(!peer->isFastExtensionEnabled());
 
-  msg->handleEvent(new BtCancelSendingPieceEvent(1, 1024, 16*1024));
+  msg->onCancelSendingPieceEvent(BtCancelSendingPieceEvent(1, 1024, 16*1024));
 
   CPPUNIT_ASSERT(!msg->isInvalidate());
 }
 
 void BtPieceMessageTest::testToString() {
-  CPPUNIT_ASSERT_EQUAL(string("piece index=1, begin=1024, length=16384"),
-		 msg->toString());
+  CPPUNIT_ASSERT_EQUAL(std::string("piece index=1, begin=1024, length=16384"),
+		       msg->toString());
 }
+
+} // namespace aria2
