@@ -4,11 +4,10 @@
 
 #include <cppunit/extensions/HelperMacros.h>
 
-#include "PeerMessageUtil.h"
+#include "bittorrent_helper.h"
 #include "Peer.h"
 #include "FileEntry.h"
 #include "MockBtMessageDispatcher.h"
-#include "MockBtContext.h"
 
 namespace aria2 {
 
@@ -63,7 +62,6 @@ public:
 
   typedef SharedHandle<MockBtMessageDispatcher2> MockBtMessageDispatcher2Handle;
 
-  SharedHandle<MockBtContext> _btContext;
   SharedHandle<Peer> peer;
   SharedHandle<MockBtMessageDispatcher2> dispatcher;
   SharedHandle<BtRejectMessage> msg;
@@ -72,14 +70,10 @@ public:
     peer.reset(new Peer("host", 6969));
     peer->allocateSessionResource(1024, 1024*1024);
 
-    _btContext.reset(new MockBtContext());
-    _btContext->setInfoHash((const unsigned char*)"12345678901234567890");
-
     dispatcher.reset(new MockBtMessageDispatcher2());
 
     msg.reset(new BtRejectMessage());
     msg->setPeer(peer);
-    msg->setBtContext(_btContext);
     msg->setIndex(1);
     msg->setBegin(16);
     msg->setLength(32);
@@ -92,10 +86,10 @@ CPPUNIT_TEST_SUITE_REGISTRATION(BtRejectMessageTest);
 
 void BtRejectMessageTest::testCreate() {
   unsigned char msg[17];
-  PeerMessageUtil::createPeerMessageString(msg, sizeof(msg), 13, 16);
-  PeerMessageUtil::setIntParam(&msg[5], 12345);
-  PeerMessageUtil::setIntParam(&msg[9], 256);
-  PeerMessageUtil::setIntParam(&msg[13], 1024);
+  bittorrent::createPeerMessageString(msg, sizeof(msg), 13, 16);
+  bittorrent::setIntParam(&msg[5], 12345);
+  bittorrent::setIntParam(&msg[9], 256);
+  bittorrent::setIntParam(&msg[13], 1024);
   SharedHandle<BtRejectMessage> pm = BtRejectMessage::create(&msg[4], 13);
   CPPUNIT_ASSERT_EQUAL((uint8_t)16, pm->getId());
   CPPUNIT_ASSERT_EQUAL((size_t)12345, pm->getIndex());
@@ -105,7 +99,7 @@ void BtRejectMessageTest::testCreate() {
   // case: payload size is wrong
   try {
     unsigned char msg[18];
-    PeerMessageUtil::createPeerMessageString(msg, sizeof(msg), 14, 16);
+    bittorrent::createPeerMessageString(msg, sizeof(msg), 14, 16);
     BtRejectMessage::create(&msg[4], 14);
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(...) {
@@ -113,7 +107,7 @@ void BtRejectMessageTest::testCreate() {
   // case: id is wrong
   try {
     unsigned char msg[17];
-    PeerMessageUtil::createPeerMessageString(msg, sizeof(msg), 13, 17);
+    bittorrent::createPeerMessageString(msg, sizeof(msg), 13, 17);
     BtRejectMessage::create(&msg[4], 13);
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(...) {
@@ -126,10 +120,10 @@ void BtRejectMessageTest::testGetMessage() {
   msg.setBegin(256);
   msg.setLength(1024);
   unsigned char data[17];
-  PeerMessageUtil::createPeerMessageString(data, sizeof(data), 13, 16);
-  PeerMessageUtil::setIntParam(&data[5], 12345);
-  PeerMessageUtil::setIntParam(&data[9], 256);
-  PeerMessageUtil::setIntParam(&data[13], 1024);
+  bittorrent::createPeerMessageString(data, sizeof(data), 13, 16);
+  bittorrent::setIntParam(&data[5], 12345);
+  bittorrent::setIntParam(&data[9], 256);
+  bittorrent::setIntParam(&data[13], 1024);
   CPPUNIT_ASSERT(memcmp(msg.getMessage(), data, 17) == 0);
 }
 
