@@ -1,8 +1,10 @@
 #include "BtUnchokeMessage.h"
-#include "PeerMessageUtil.h"
+#include "bittorrent_helper.h"
+#include "Peer.h"
+#include <cstring>
 #include <cppunit/extensions/HelperMacros.h>
 
-using namespace std;
+namespace aria2 {
 
 class BtUnchokeMessageTest:public CppUnit::TestFixture {
 
@@ -30,14 +32,14 @@ CPPUNIT_TEST_SUITE_REGISTRATION(BtUnchokeMessageTest);
 
 void BtUnchokeMessageTest::testCreate() {
   unsigned char msg[5];
-  PeerMessageUtil::createPeerMessageString(msg, sizeof(msg), 1, 1);
-  BtUnchokeMessageHandle pm = BtUnchokeMessage::create(&msg[4], 1);
-  CPPUNIT_ASSERT_EQUAL((int8_t)1, pm->getId());
+  bittorrent::createPeerMessageString(msg, sizeof(msg), 1, 1);
+  SharedHandle<BtUnchokeMessage> pm = BtUnchokeMessage::create(&msg[4], 1);
+  CPPUNIT_ASSERT_EQUAL((uint8_t)1, pm->getId());
 
   // case: payload size is wrong
   try {
     unsigned char msg[6];
-    PeerMessageUtil::createPeerMessageString(msg, sizeof(msg), 2, 1);
+    bittorrent::createPeerMessageString(msg, sizeof(msg), 2, 1);
     BtUnchokeMessage::create(&msg[4], 2);
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(...) {
@@ -45,7 +47,7 @@ void BtUnchokeMessageTest::testCreate() {
   // case: id is wrong
   try {
     unsigned char msg[5];
-    PeerMessageUtil::createPeerMessageString(msg, sizeof(msg), 1, 2);
+    bittorrent::createPeerMessageString(msg, sizeof(msg), 1, 2);
     BtUnchokeMessage::create(&msg[4], 1);
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(...) {
@@ -55,33 +57,37 @@ void BtUnchokeMessageTest::testCreate() {
 void BtUnchokeMessageTest::testGetMessage() {
   BtUnchokeMessage msg;
   unsigned char data[5];
-  PeerMessageUtil::createPeerMessageString(data, sizeof(data), 1, 1);
+  bittorrent::createPeerMessageString(data, sizeof(data), 1, 1);
   CPPUNIT_ASSERT(memcmp(msg.getMessage(), data, 5) == 0);
 }
 
 void BtUnchokeMessageTest::testDoReceivedAction() {
-  PeerHandle peer = new Peer("host", 6969, 16*1024, 256*1024);
-  peer->peerChoking = true;
+  SharedHandle<Peer> peer(new Peer("host", 6969));
+  peer->allocateSessionResource(1024, 1024*1024);
+  peer->peerChoking(true);
   BtUnchokeMessage msg;
   msg.setPeer(peer);
 
-  CPPUNIT_ASSERT(peer->peerChoking);
+  CPPUNIT_ASSERT(peer->peerChoking());
   msg.doReceivedAction();
-  CPPUNIT_ASSERT(!peer->peerChoking);
+  CPPUNIT_ASSERT(!peer->peerChoking());
 }
 
 void BtUnchokeMessageTest::testOnSendComplete() {
-  PeerHandle peer = new Peer("host", 6969, 16*1024, 256*1024);
-  peer->amChoking = true;
+  SharedHandle<Peer> peer(new Peer("host", 6969));
+  peer->allocateSessionResource(1024, 1024*1024);
+  peer->amChoking(true);
   BtUnchokeMessage msg;
   msg.setPeer(peer);
 
-  CPPUNIT_ASSERT(peer->amChoking);
+  CPPUNIT_ASSERT(peer->amChoking());
   msg.onSendComplete();
-  CPPUNIT_ASSERT(!peer->amChoking);
+  CPPUNIT_ASSERT(!peer->amChoking());
 }
 
 void BtUnchokeMessageTest::testToString() {
   BtUnchokeMessage msg;
-  CPPUNIT_ASSERT_EQUAL(string("unchoke"), msg.toString());
+  CPPUNIT_ASSERT_EQUAL(std::string("unchoke"), msg.toString());
 }
+
+} // namespace aria2
