@@ -3,11 +3,10 @@
 #include <cppunit/extensions/HelperMacros.h>
 
 #include "DHTNode.h"
-#include "DHTUtil.h"
 #include "Exception.h"
-#include "Util.h"
+#include "util.h"
 #include "DHTBucket.h"
-#include "PeerMessageUtil.h"
+#include "bittorrent_helper.h"
 #include "Peer.h"
 #include "bencode.h"
 
@@ -35,18 +34,18 @@ void DHTGetPeersReplyMessageTest::testGetBencodedMessage()
   SharedHandle<DHTNode> remoteNode(new DHTNode());
 
   unsigned char tid[DHT_TRANSACTION_ID_LENGTH];
-  DHTUtil::generateRandomData(tid, DHT_TRANSACTION_ID_LENGTH);
+  util::generateRandomData(tid, DHT_TRANSACTION_ID_LENGTH);
   std::string transactionID(&tid[0], &tid[DHT_TRANSACTION_ID_LENGTH]);
 
   std::string token = "token";
 
   DHTGetPeersReplyMessage msg(localNode, remoteNode, token, transactionID);
 
-  bencode::BDE dict = bencode::BDE::dict();
+  BDE dict = BDE::dict();
   dict["t"] = transactionID;
-  dict["y"] = bencode::BDE("r");
-  bencode::BDE rDict = bencode::BDE::dict();
-  rDict["id"] = bencode::BDE(localNode->getID(), DHT_ID_LENGTH);
+  dict["y"] = BDE("r");
+  BDE rDict = BDE::dict();
+  rDict["id"] = BDE(localNode->getID(), DHT_ID_LENGTH);
   rDict["token"] = token;
   dict["r"] = rDict;
   {
@@ -54,11 +53,11 @@ void DHTGetPeersReplyMessageTest::testGetBencodedMessage()
     SharedHandle<DHTNode> nodes[8];
     for(size_t i = 0; i < DHTBucket::K; ++i) {
       nodes[i].reset(new DHTNode());
-      nodes[i]->setIPAddress("192.168.0."+Util::uitos(i+1));
+      nodes[i]->setIPAddress("192.168.0."+util::uitos(i+1));
       nodes[i]->setPort(6881+i);
       
       unsigned char buf[6];
-      CPPUNIT_ASSERT(PeerMessageUtil::createcompact
+      CPPUNIT_ASSERT(bittorrent::createcompact
 		     (buf, nodes[i]->getIPAddress(), nodes[i]->getPort()));
       compactNodeInfo +=
 	std::string(&nodes[i]->getID()[0], &nodes[i]->getID()[DHT_ID_LENGTH])+
@@ -71,18 +70,19 @@ void DHTGetPeersReplyMessageTest::testGetBencodedMessage()
 
     rDict["nodes"] = compactNodeInfo;
 
-    CPPUNIT_ASSERT_EQUAL(Util::urlencode(bencode::encode(dict)),
-			 Util::urlencode(msgbody));
+    CPPUNIT_ASSERT_EQUAL(util::urlencode(bencode::encode(dict)),
+			 util::urlencode(msgbody));
   }
   rDict.removeKey("nodes");
   {
     std::deque<SharedHandle<Peer> > peers;
-    bencode::BDE valuesList = bencode::BDE::list();
+    BDE valuesList = BDE::list();
     for(size_t i = 0; i < 4; ++i) {
-      SharedHandle<Peer> peer(new Peer("192.168.0."+Util::uitos(i+1), 6881+i));
+      SharedHandle<Peer> peer(new Peer("192.168.0."+util::uitos(i+1), 6881+i));
       unsigned char buffer[6];
-      CPPUNIT_ASSERT(PeerMessageUtil::createcompact(buffer, peer->ipaddr, peer->port));
-      valuesList << bencode::BDE(buffer, sizeof(buffer));
+      CPPUNIT_ASSERT(bittorrent::createcompact
+		     (buffer, peer->ipaddr, peer->port));
+      valuesList << BDE(buffer, sizeof(buffer));
       peers.push_back(peer);
     }
     rDict["values"] = valuesList;
@@ -90,8 +90,8 @@ void DHTGetPeersReplyMessageTest::testGetBencodedMessage()
     msg.setValues(peers);
     std::string msgbody  = msg.getBencodedMessage();
 
-    CPPUNIT_ASSERT_EQUAL(Util::urlencode(bencode::encode(dict)),
-			 Util::urlencode(msgbody));
+    CPPUNIT_ASSERT_EQUAL(util::urlencode(bencode::encode(dict)),
+			 util::urlencode(msgbody));
   }
 }
 
