@@ -1,9 +1,11 @@
 #include "MessageDigestHelper.h"
-#include "Util.h"
-#include "DefaultDiskWriter.h"
+
 #include <cppunit/extensions/HelperMacros.h>
 
-using namespace std;
+#include "util.h"
+#include "DefaultDiskWriter.h"
+
+namespace aria2 {
 
 class MessageDigestHelperTest:public CppUnit::TestFixture {
 
@@ -27,24 +29,27 @@ public:
 CPPUNIT_TEST_SUITE_REGISTRATION( MessageDigestHelperTest );
 
 void MessageDigestHelperTest::testDigestDiskWriter() {
-  DiskWriterHandle diskio = new DefaultDiskWriter();
-  diskio->openExistingFile("4096chunk.txt");
-  CPPUNIT_ASSERT_EQUAL(string("608cabc0f2fa18c260cafd974516865c772363d5"),
+  SharedHandle<DefaultDiskWriter> diskio
+    (new DefaultDiskWriter("4096chunk.txt"));
+  diskio->openExistingFile();
+  CPPUNIT_ASSERT_EQUAL(std::string("608cabc0f2fa18c260cafd974516865c772363d5"),
 		       MessageDigestHelper::digest("sha1", diskio, 0, 4096));
 
-  CPPUNIT_ASSERT_EQUAL(string("7a4a9ae537ebbbb826b1060e704490ad0f365ead"),
+  CPPUNIT_ASSERT_EQUAL(std::string("7a4a9ae537ebbbb826b1060e704490ad0f365ead"),
 		       MessageDigestHelper::digest("sha1", diskio, 5, 100));
 }
 
 void MessageDigestHelperTest::testDigestFilename()
 {
-  CPPUNIT_ASSERT_EQUAL(string("608cabc0f2fa18c260cafd974516865c772363d5"),
+  CPPUNIT_ASSERT_EQUAL(std::string("608cabc0f2fa18c260cafd974516865c772363d5"),
 		       MessageDigestHelper::digest("sha1", "4096chunk.txt"));
 }
 
 void MessageDigestHelperTest::testDigestData()
 {
-  string data = "aria2";
-  CPPUNIT_ASSERT_EQUAL(string("f36003f22b462ffa184390533c500d8989e9f681"),
+  std::string data = "aria2";
+  CPPUNIT_ASSERT_EQUAL(std::string("f36003f22b462ffa184390533c500d8989e9f681"),
 		       MessageDigestHelper::digest("sha1", data.c_str(), data.size()));
 }
+
+} // namespace aria2
