@@ -5,7 +5,7 @@
 #include <cppunit/extensions/HelperMacros.h>
 
 #include "prefs.h"
-#include "SingleFileDownloadContext.h"
+#include "DownloadContext.h"
 #include "RequestGroup.h"
 #include "Option.h"
 #include "DownloadResult.h"
@@ -43,28 +43,26 @@ CPPUNIT_TEST_SUITE_REGISTRATION( RequestGroupManTest );
 
 void RequestGroupManTest::testIsSameFileBeingDownloaded()
 {
-  std::deque<std::string> uris;
-  uris.push_back("http://localhost/aria2.tar.bz2");
-  SharedHandle<RequestGroup> rg1(new RequestGroup(_option, uris));
-  SharedHandle<RequestGroup> rg2(new RequestGroup(_option, uris));
+  SharedHandle<RequestGroup> rg1(new RequestGroup(_option));
+  SharedHandle<RequestGroup> rg2(new RequestGroup(_option));
 
-  SharedHandle<SingleFileDownloadContext> dctx1
-    (new SingleFileDownloadContext(0, 0, "aria2.tar.bz2"));
-  SharedHandle<SingleFileDownloadContext> dctx2
-    (new SingleFileDownloadContext(0, 0, "aria2.tar.bz2"));
+  SharedHandle<DownloadContext> dctx1
+    (new DownloadContext(0, 0, "aria2.tar.bz2"));
+  SharedHandle<DownloadContext> dctx2
+    (new DownloadContext(0, 0, "aria2.tar.bz2"));
 
   rg1->setDownloadContext(dctx1);
   rg2->setDownloadContext(dctx2);
 
-  RequestGroups rgs;
-  rgs.push_back(rg1);
-  rgs.push_back(rg2);
+  RequestGroupMan gm(std::deque<SharedHandle<RequestGroup> >(), 1,
+		     _option.get());
 
-  RequestGroupMan gm(rgs, 1, _option.get());
+  gm.addRequestGroup(rg1);
+  gm.addRequestGroup(rg2);
   
   CPPUNIT_ASSERT(gm.isSameFileBeingDownloaded(rg1.get()));
 
-  dctx2->setFilename("aria2.tar.gz");
+  dctx2->getFirstFileEntry()->setPath("aria2.tar.gz");
 
   CPPUNIT_ASSERT(!gm.isSameFileBeingDownloaded(rg1.get()));
 
